//SLAMM SOURCE CODE copyright (c) 2009 - 2012 Warren Pinnacle Consulting, Inc.
//Code Use and Redistribution is Subject to Licensing, SEE SLAMM_License.txt
//

unit SiteEdits;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Main, StdCtrls, Grids, Menus, ExtCtrls, Clipbrd, Global, SLR6,
  Buttons;

Const NUMRowsOrig = 21;      // original parameters
      NUMRowsOverwash = 6;   // overwash parameters
      NUMRowsAccr = 12;      // accretion model rows

type
  TSiteEditForm = class(TForm)
    OKBtn: TButton;
    CancelBtn: TButton;
    ModLabel: TLabel;
    TSGrid: TStringGrid;
    Label1: TLabel;
    ShowOverwash: TCheckBox;
    ShowAccr: TCheckBox;
    ExcelExport: TButton;
    SaveDialog1: TSaveDialog;
    HelpButton: TBitBtn;
    SaveButton: TButton;
    procedure ComboBox1Change(Sender: TObject);
    procedure CancelBtnClick(Sender: TObject);
    procedure TSGridExit(Sender: TObject);
    procedure TSGridSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure TSGridSetEditText(Sender: TObject; ACol, ARow: Integer;
      const Value: string);
    procedure FormCreate(Sender: TObject);
    procedure CostEditExit(Sender: TObject);
    procedure TSGridDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect;
      State: TGridDrawState);
    procedure TSGridKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ShowOverwashClick(Sender: TObject);
    procedure ExcelExportClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
  private
    TS: TSite;
    { Private declarations }
  public
    ValStr  : String;  { [60]; }
    EditRow, EditCol : Integer;
    SaveStream: TMemoryStream;
    Procedure EditSubSites(Var PSS: TSLAMM_Simulation);
    Procedure UpdateTSGrid;
    Procedure UpdateScreen();
    { Public declarations }
  end;

var
  SiteEditForm: TSiteEditForm;

implementation

Uses Elev_Analysis, System.UITypes;

{$R *.dfm}

procedure TSiteEditForm.TSGridDrawCell(Sender: TObject; ACol, ARow: Integer;
  Rect: TRect; State: TGridDrawState);
begin
  If (ARow=0) or (ACol=0) then
      with TSGrid.Canvas do
        begin
          Font.Style := [fsbold];
    //      TextRect(Rect, Rect.Left + 2, Rect.Top + 2, TSGrid.Cells[ACol, ARow]);    Doesn't work in XE3, trying to make font bold
        end;

end;

procedure TSiteEditForm.TSGridExit(Sender: TObject);

Var AM, R, C: Integer;
    V: String;
    TSS: TSubSite;

    Function RemovePct(Str: String):String;
    Var P: Integer;
    Begin
      Result := Str;
      P := Pos('%',Result);
      While P>0 Do
        Begin
          Delete(Result,P,1);
          P := Pos('%',Result);
        End;
    End;

begin
 CancelBtn.Enabled := True;

 IF EditRow < 1 then exit;
 If EditCol < 1 then exit;

  If (Trim(ValStr) = '~') then exit;
  R := EditRow; C := EditCol; V := ValStr;
  EditRow := -1; EditCol := -1; ValStr := '~';

  If C=1 then TSS := TS.GlobalSite
         else TSS := TS.SubSites[C-2];

  If v='' then exit;
  With TSS do
    Try
      Case R of
        1: Description := V;
        2: NWI_Photo_Date := Trunc(StrToFloat(V));
        3: DEM_Date := Trunc(StrToFloat(V));
        4: Begin
             V := Lowercase(V);
             If v='' then exit;
             If V[1] = 's' then Direction_OffShore := Southerly
                else if Pos('n',V) > 0 then Direction_OffShore := Northerly
                else if Pos('w',V) > 0 then Direction_OffShore := Westerly
                else Direction_OffShore := Easterly;
           End;
        5: Historic_Trend := StrToFloat(V);
        6: NAVD88MTL_Correction := StrToFloat(V);
        7: GTideRange := StrToFloat(V);
        8: SaltElev := StrToFloat(V);
        9: MarshErosion := StrToFloat(V);
        10: SwampErosion := StrToFloat(V);
        11: TFlatErosion := StrToFloat(V);
        12: FixedRegFloodAccr  := StrToFloat(V);         // Note edits here also need to be made in UncertDefn
        13: FixedIrregFloodAccr  := StrToFloat(V);
        14: FixedTideFreshAccr  := StrToFloat(V);

        15: InlandFreshAccr := StrToFloat(V);
        16: MangroveAccr := StrToFloat(V);
        17: TSwampAccr := StrToFloat(V);
        18: SwampAccr := StrToFloat(V);

        19: Fixed_TF_Beach_Sed := StrToFloat(V);
        20: Freq_BigStorm := StrToFloat(V);
        21: Begin
              v := Lowercase(v);
              If (v[1]='f') or (v[1] = '0')
                then Use_Preprocessor:= False
                else Use_Preprocessor:= True;
            End;
      End; {Case}

    If ShowOverWash.Checked
     then Case R of
      22: OW_Max_Width_Overwash := StrToFloat(V);
      23: OW_Beach_to_Ocean := StrToFloat(V);
      24: OW_Dryland_to_Beach := StrToFloat(V);
      25: OW_Est_to_Beach := StrToFloat(V);
      26: OW_Marsh_Pct_Loss := StrToFloat(RemovePct(V));
      27: OW_Mang_Pct_Loss := StrToFloat(RemovePct(V));
      End; {Case}


    If (R>NUMRowsOrig+NUMRowsOverwash) or ((R>NUMRowsOrig) and Not ShowOverWash.Checked) then
      Begin {editing accretion model parameters}
        R := R-NUMRowsOrig;
        If ShowOverWash.Checked then R := R-NUMRowsOverwash;  {normalize to row 1-44}
        AM := (R-1) Div NUMRowsAccr;  {Accretion model edited, 0 to 3}
        R := R-(NUMRowsAccr*AM); {normalize to row 1-NUMRowsAccr}
        Case R of
          1: Begin
                v := Lowercase(v);
                If (v[1]='f') or (v[1] = '0')
                  then UseAccrModel[AM]:= False
                  else UseAccrModel[AM]:= True;
              End;
          2: MaxAccr[AM] := (StrToFloat(V));
          3: MinAccr[AM] := (StrToFloat(V));
          4: AccrA[AM] := (StrToFloat(V));
          5: AccrB[AM] := (StrToFloat(V));
          6: AccrC[AM] := (StrToFloat(V));
          7: DistEffectMax[AM] := (StrToFloat(V));
          8: Dmin[AM] := StrToFloat(V);
          9: SalinityTMax[AM] := StrToFloat(V);
          10: STMaxZone[AM]     := StrToFloat(V);
          11: SNonTMax[AM]  := StrToFloat(V);
          12: AccrNotes[AM] := (V);
        End; {Case}
     End; {if}

    Except
    End;

  UpdateScreen;
end;

procedure TSiteEditForm.TSGridKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
// Paste
    Procedure PasteIntoGrid;
    var
       Grect:TGridRect;
       S,CS,F:String;
       L,R,C:Byte;
    begin
       GRect:=TSGrid.Selection;
       L:=GRect.Left; R:=GRect.Top;
       S:=ClipBoard.AsText;
       R:=R-1 ;
       while Pos(#13,S)>0 do
       begin
           R:=R+1;
           C:=L-1;
           CS:= Copy(S,1,Pos(#13,S));
           while Pos(#9,CS)>0 do
           begin
               C:=C+1;
               if (C<=TSGrid.ColCount-1)and (R<=TSGrid.RowCount-1) then
                   ValStr  :=Copy(CS,1,Pos(#9,CS)-1);
                   EditRow := R;
                   EditCol := C;
                   TSGridExit(nil);
                   F:= Copy(CS,1,Pos(#9,CS)-1);
               Delete(CS,1,Pos(#9,CS));
           end;
           if (C<=TSGrid.ColCount-1)and (R<=TSGrid.RowCount-1) then
              Begin
                ValStr  := Copy(CS,1,Pos(#13,CS)-1);
                EditRow := R;
                EditCol := C+1;
                TSGridExit(nil);
              End;
//         TSGrid.Cells[C+1,R]:=Copy(CS,1,Pos(#13,CS)-1);

           Delete(S,1,Pos(#13,S));
           if Copy(S,1,1)=#10 then
           Delete(S,1,1);
       end;
    end;

begin

    If ((ssCtrl in Shift) AND (Key = ord('V')))  then
      Begin
        PasteIntoGrid;
        Key := 0;
      End;

end;

procedure TSiteEditForm.TSGridSelectCell(Sender: TObject; ACol,
  ARow: Integer; var CanSelect: Boolean);
begin
//  If ARow + 1 = TSGrid.RowCount then CanSelect := False;
  TSGridExit(nil);
end;

procedure TSiteEditForm.TSGridSetEditText(Sender: TObject; ACol,
  ARow: Integer; const Value: string);
begin
  ValStr  := Value;
  EditRow := ARow;
  EditCol := ACol;

end;

Procedure TSiteEditForm.CancelBtnClick(Sender: TObject);
begin
  If MessageDlg('Cancel all Changes?',mtconfirmation,[mbyes,mbno],0) = MRyes then
    ModalResult := MRCancel;
end;

Procedure TSiteEditForm.ComboBox1Change(Sender: TObject);
begin
  UpdateScreen;
end;


procedure TSiteEditForm.CostEditExit(Sender: TObject);
begin
  UpdateTSGrid;
end;

procedure TSiteEditForm.EditSubSites(var PSS: TSLAMM_Simulation);

begin

  TSText := False;
  SaveStream := TMemoryStream.Create;
  GlobalTS := SaveStream;
  PSS.Site.Store(TStream(SaveStream));  {save for backup in case of cancel click}
  TS := PSS.Site;

  UpdateTSGrid;

  SaveStream.Seek(0, soFromBeginning); {Go to beginning of stream}
  If ShowModal = MRCancel
     then PSS.Site.Load(VersionNum,TStream(SaveStream))
     else if CancelBtn.Enabled then PSS.Changed := True;
  SaveStream.Free;

end;

procedure TSiteEditForm.ExcelExportClick(Sender: TObject);
Var SaveName: String;
Begin
      // Create save dialog and set it options
      with SaveDialog1 do
      begin
         DefaultExt := 'xlsx' ;
         Filter := 'Excel files (*.xlsx)|*.xlsx|All files (*.*)|*.*' ;
         Options := [ofOverwritePrompt,ofPathMustExist,ofNoReadOnlyReturn,ofHideReadOnly] ;
         Title := 'Please Specify an Excel File into which to Save this Table:';
      end ;

   if not SaveDialog1.Execute then exit;

   SaveName := SaveDialog1.FileName;
   If FileExists(SaveDialog1.FileName) then If Not DeleteFile(SaveName)
     then Begin
            MessageDlg('Cannot gain exclusive access to the file to overwrite it.',mtError,[mbOK],0);
            Exit;
          End;

 SaveAsExcelFile(TSGrid, 'Elev Analysis', SaveName);
end;

procedure TSiteEditForm.FormCreate(Sender: TObject);
begin
  EditRow := -1; EditCol := -1; ValStr := '~';
end;



procedure TSiteEditForm.HelpButtonClick(Sender: TObject);
begin
   Application.HelpContext(1009 );  //'EditSites.html');
end;

procedure TSiteEditForm.SaveButtonClick(Sender: TObject);
begin
MainForm.SaveSimClick(Sender);
CancelBtn.Enabled := False;
end;

procedure TSiteEditForm.ShowOverwashClick(Sender: TObject);
begin
  UpdateTSGrid;
end;

Procedure TSiteEditForm.UpdateScreen;
Begin
  UpdateTSGrid;
End;


procedure TSiteEditForm.UpdateTSGrid;
Var nRows, i,j: Integer;

    Procedure AddColumn(SubS: TSubSite);
    Var i, NextRow: Integer;
    Begin
     With SubS do with TSGrid do
      Begin
        Rows[1].Add(Description);
        Rows[2].Add(IntToStr(NWI_Photo_Date));
        Rows[3].Add(IntToStr(DEM_Date));
        Case SubS.Direction_OffShore of
           Southerly: Rows[4].Add('South');
           Northerly: Rows[4].Add('North');
           Easterly: Rows[4].Add('East');
           Westerly: Rows[4].Add('West');
           end{Case};
        Rows[5].Add(FloatToStrf(Historic_Trend,ffgeneral,8,4));
        Rows[6].Add(FloatToStrf(NAVD88MTL_Correction,ffgeneral,8,4));
        Rows[7].Add((FloatToStrf(GTideRange,ffgeneral,8,4)));
        Rows[8].Add((FloatToStrf(SaltElev,ffgeneral,8,4)));
        Rows[9].Add((FloatToStrf(MarshErosion,ffgeneral,8,4)));
        Rows[10].Add((FloatToStrf(SwampErosion,ffgeneral,8,4)));
        Rows[11].Add((FloatToStrf(TFlatErosion,ffgeneral,8,4)));
        Rows[12].Add((FloatToStrf(FixedRegFloodAccr ,ffgeneral,8,4)));
        Rows[13].Add((FloatToStrf(FixedIrregFloodAccr ,ffgeneral,8,4)));
        Rows[14].Add((FloatToStrf(FixedTideFreshAccr ,ffgeneral,8,4)));

        Rows[15].Add((FloatToStrf(InlandFreshAccr ,ffgeneral,8,4)));
        Rows[16].Add((FloatToStrf(MangroveAccr ,ffgeneral,8,4)));
        Rows[17].Add((FloatToStrf(TSwampAccr ,ffgeneral,8,4)));
        Rows[18].Add((FloatToStrf(SwampAccr ,ffgeneral,8,4)));


        Rows[19].Add((FloatToStrf(Fixed_TF_Beach_Sed,ffgeneral,8,4)));
        Rows[20].Add(FloatToStrf(Freq_BigStorm,ffgeneral,8,4));
        If Use_Preprocessor then Rows[21].Add('True')
                            else Rows[21].Add('False');
        NextRow := 22;

        If Showoverwash.Checked then
          Begin
            Rows[22].Add((FloatToStrf(OW_Max_Width_Overwash,ffgeneral,8,4)));
            Rows[23].Add((FloatToStrf(OW_Beach_to_Ocean,ffgeneral,8,4)));
            Rows[24].Add((FloatToStrf(OW_Dryland_to_Beach,ffgeneral,8,4)));
            Rows[25].Add((FloatToStrf(OW_Est_to_Beach,ffgeneral,8,4)));
            Rows[26].Add(FloatToStrF(OW_Marsh_Pct_Loss,ffgeneral,4,2) +'%');
            Rows[27].Add(FloatToStrF(OW_Mang_Pct_Loss,ffgeneral,4,2) +'%');
            NextRow := 28;
          End;

        If ShowAccr.Checked then
         For i := 0 to N_Accr_Models-1 do
          Begin
            If UseAccrModel[i] then Rows[NextRow].Add('True')
                               else Rows[NextRow].Add('False');
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(MaxAccr[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(MinAccr[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(AccrA[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(AccrB[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(AccrC[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(DistEffectMax[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(Dmin[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(SalinityTMax[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(STMaxZone[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add((FloatToStrf(SNonTMax[i],ffgeneral,8,4)));
            Inc(NextRow);
            Rows[NextRow].Add(AccrNotes[i]);
            Inc(NextRow);
          End;
      End; {with SubS}
    End;

Var NextRow: INteger;
begin
  With TSGrid do
    Begin
      If ShowOverwash.Checked then NRows := 27
                              else NRows := 21;
      If ShowAccr.Checked then NRows := NRows + 12 * N_ACCR_MODELS;
      RowCount := NRows+1;  {plus header}
      ColCount := TS.NSubSites + 2;  {plus header and global site}

      ColWidths[0]:=205;
//      ColWidths[ColCount-1]:=130;

      For i := 0 to RowCount-1 do
        Rows[i].Clear;

      Rows[0].Add('Parameter');
      Rows[0].Add('Global');

      For i := 1 to TS.NSubSites do
        Rows[0].Add('SubSite '+IntToSTr(i));

      Rows[1].Add('Description');
      Rows[2].Add('NWI Photo Date (YYYY)');
      Rows[3].Add('DEM Date (YYYY)');
      Rows[4].Add('Direction Offshore [n,s,e,w]');
      Rows[5].Add('Historic Trend (mm/yr)');
      Rows[6].Add('MTL-NAVD88 (m)');
      Rows[7].Add('GT Great Diurnal Tide Range (m)');
      Rows[8].Add('Salt Elev. (m above MTL)');
      Rows[9].Add('Marsh Erosion (horz. m /yr)');
      Rows[10].Add('Swamp Erosion (horz. m /yr)');
      Rows[11].Add('T.Flat Erosion (horz. m /yr)');
      Rows[12].Add('Reg.-Flood Marsh Accr (mm/yr)');
      Rows[13].Add('Irreg.-Flood Marsh Accr (mm/yr)');
      Rows[14].Add('Tidal-Fresh Marsh Accr (mm/yr)');

      Rows[15].Add('Inland-Fresh Marsh Accr (mm/yr)');
      Rows[16].Add('Mangrove Accr (mm/yr)');
      Rows[17].Add('Tidal Swamp Accr (mm/yr)');
      Rows[18].Add('Swamp Accretion (mm/yr)');

      Rows[19].Add('Beach Sed. Rate (mm/yr)');
      Rows[20].Add('Freq. Overwash (years)');
      Rows[21].Add('Use Elev Pre-processor [True,False]');
      NextRow := 22;

      If Showoverwash.Checked then
        Begin
          Rows[22].Add('Max Width Overwash (m)');
          Rows[23].Add('Beach to Ocean Overwash (m)');
          Rows[24].Add('Dryland to Beach Overwash (m)');
          Rows[25].Add('Estuary to Beach Overwash (m)');
          Rows[26].Add('Marsh Pct Loss Overwash (%)');
          Rows[27].Add('Mang. Pct. Loss Overwash (%)');
          NextRow := 28;
        End;

      If ShowAccr.Checked then
       For i := 0 to N_Accr_Models-1 do
        Begin
          Rows[NextRow].Add(AccrNames[i]+' Use Model [True,False]');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Max. Accr. (mm/year)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Min. Accr. (mm/year)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Elev a coeff. (cubic)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Elev b coeff. (square)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Elev c coeff. (linear)');
          Inc(NextRow);

          Rows[NextRow].Add(AccrNames[i]+' D.Effect Max (meters)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' D min. (unitless)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Salinity Turb. Max (ppt)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Turb. Max Zone (ppt)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' S. Non T.Max (unitless)');
          Inc(NextRow);
          Rows[NextRow].Add(AccrNames[i]+' Notes');
          Inc(NextRow);
        End;

      For j := 0 to TS.NSubSites do
        Begin
          If j=0 then AddColumn(TS.GlobalSite)
                 else AddColumn(TS.SubSites[j-1]);
        End;

    End; {With TSGrid}
end;

end.
