//SLAMM SOURCE CODE copyright (c) 2009 - 2012 Warren Pinnacle Consulting, Inc.
//Code Use and Redistribution is Subject to Licensing, SEE SLAMM_License.txt
//

(*********************************)
(*   summarize cell categories   *)
(*********************************)
(*****************************)
(*  create CSV file          *)
(*****************************)
PROCEDURE TSLAMM_Simulation.                    //called at end of simulation
          CSV_file_save(datary:   OutputArray;
                        rowlabel: str_vector1;
                        collabel: str_vector;
                        numrow:   integer;
                        numcol:   integer;
                        fnm: String);

VAR
  F: text;
  SLLabel: String;
  Var m,n: Integer;
  WriteError: Boolean;
  SecondaryFileName: String;

     {=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=}
     Function ProtLabel: String;
     Begin
       With UncertSetup do
        If SensAnalysis and (UncSens_Iter>0)
          then
            Begin
              Result := '"'+ IntToStr(UncSens_Iter);
            End
          else
            IF ProtectAll
                THEN Result := '"Protect All Dry'
                ELSE IF ProtectDeveloped
                        THEN Result := '"Protect Developed Dry'
                        ELSE Result := '"Protect None';

        If Not IncludeDikes then Result := Result + ' No Dikes';

        Result := Result + '",';
     End;
     {=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=}
     Procedure WriteSecondaryFile;  // Jan 8, 2010 write secondary CSV output to Industrial Economics Specs
     Var Appending: Boolean;
         RunDate: TDateTime;
         isites, irows,j: integer;
     Begin
        If OutputFileN <> '' then SecondaryFileName := OutputFileN + '.CSV'
                             else SecondaryFileName := ElevFileN + '_OUT.CSV';

        Repeat
          WriteError := False;
          Appending := False;
        Try
          Appending := FileExists(SecondaryFileName);
          AssignFile(F,SecondaryFileName);
          If Appending
                then Append(F)
                else Rewrite(F);
        Except
          If MessageDlg('Error writing to file '+SecondaryFileName+'.  Retry?',mterror,[mbyes,mbno],0)
            = mryes then WriteError := True
                    else Exit;
        End;
        Until Not WriteError;

        If Not Appending then
          Writeln(F,'"Site Desc.","Scenario","Parameters","Year","Protection","SLAMMCode","Hectares","SLAMMText","SLR (eustatic)","RunDate"');

        RunDate := Now();

        For isites := 0 to Site.NOutputSites + Site.MaxROS do
         FOR irows := 0 TO numrow-1 DO
          FOR j := 1 TO numcol+10 DO
           BEGIN
              If isites=0 then Write(F,'"',Site.GlobalSite.Description,'",')
                          else If (isites <= Site.NOutputSites)
                            then Write(F,'"',Site.OutputSites[isites-1].Description,'",')
                            else Write(F,'"Raster ',IntToStr(Isites-Site.NOutputSites),'",');

              If Running_Fixed then Write(F,'"Fixed","',LabelFixed[FixedScen],'",') else
              If Running_Custom then Write(F,'"Custom","',FloatToStrF(CustomSLR ,ffgeneral,4,2)+ ' meters ','",')
                               else Write(F,'"',LabelIPCC[IPCCSLRate],'","',LabelIPCCEst[IPCCSLEst],'",');

              Write(F,'"',rowlabel[irows],'",');  {write year}
              Write(F,ProtLabel);  {write protection}
              If j<27 then Write(F,j,',')  {SLAMM Code}
                      else Write(F,'NA,');
              Write(F,datary[isites,irows,j]:n:m,','); {Hectares}
              Write(F,'"',collabel[j],'",'); {Slamm Text}
              Write(F,datary[isites,irows,0]:n:m,',');  {Eustatic SLR}
              Writeln(F,DateTimeToStr(RunDate));

           End;
        CloseFile(F);
     End;
     {=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=}

Var
  isites, irows,j: integer;

BEGIN
  n:=10;
  m:=4;
  AssignFile(F,fnm);

  Repeat
    WriteError := False;
  Try
    If BatchMode and FileStarted then Append(F)
                                 else Rewrite(F);
  Except
    If MessageDlg('Error writing to file '+fnm+'.  Retry?',mterror,[mbyes,mbno],0)
      = mryes then WriteError := True
              else Exit;
  End;

  Until Not WriteError;

  If Running_Fixed  then SLLabel := LabelFixed[FixedScen] else
  If Running_Custom then SLLabel := FloatToStrF(CustomSLR ,ffgeneral,4,2)+ ' meters '
                    else SLLabel := LabelIPCC[IPCCSLRate]+' '+LabelIPCCEst[IPCCSLEst];

  If Not BatchMode then
    Begin
      Write(F,'"','Sea Level Scenario: ',SLLabel);
      Write(F,ProtLabel);

      Writeln(F);
      WriteLn(F,'"Total Ha:",',Round(Hectares):n);
      Writeln(F);
    End;

  If Not (BatchMode and FileStarted) then
    Begin
      Write(F,'"Date","Site Desc.","Scenario","Parameters","Protection",');
      FOR J := 0 TO numcol+10 DO               {write col headers}
       if (J<>NumCol+2) and (J<>NumCol+3) then  //suppress fractal dimension & shoreline prot.
        write(F,'"',collabel[j],'",');
      Writeln(F);
    End;

  FileStarted := True;

  For isites := 0 to Site.NOutputSites + Site.MaxROS do
   FOR irows := 0 TO numrow-1 DO
    BEGIN
      Write(F,'"',rowlabel[irows],'",');

      If isites=0 then Write(F,'"',Site.GlobalSite.Description,'",')
                  else If (isites <= Site.NOutputSites)
                    then Write(F,'"',Site.OutputSites[isites-1].Description,'",')
                    else Write(F,'"Raster ',IntToStr(Isites-Site.NOutputSites),'",');

      If Running_Fixed  then Write(F,'"Fixed","',LabelFixed[FixedScen],'",') else
      If Running_Custom then Write(F,'"Custom","',FloatToStrF(CustomSLR ,ffgeneral,4,2)+ ' meters ','",')
                        else Write(F,'"',LabelIPCC[IPCCSLRate],'","',LabelIPCCEst[IPCCSLEst],'",');

      Write(F,ProtLabel);

      FOR j := 0 TO numcol+10 DO
       if (J<>NumCol+2) and (J<>NumCol+3) then  //suppress fractal dimension & shoreline prot.
         Write(F,datary[isites,irows,j]:n:m,',');

      Writeln(F);
    END;

  CloseFile(F);

  WriteSecondaryFile;

END;



PROCEDURE TSLAMM_Simulation.Summarize(VAR Map: MSArray;
                    VAR Tropical: Boolean; VAR Summary: OutputArray; Year: Integer);

//called each time step.  Create the data in the summary array that is written above
VAR
  Hectares1,
  NumCells, Aggr: double;
  CC: Category;
  i,j: Integer;

BEGIN
  Calc_Fractal(Site);

  NumCells := (Site.Rows) * (Site.Cols);

  Hectares := NumCells * Site.Scale * Site.Scale / 10000;    {total hectares}
    {ha}          {n}          {m}          {m}   {m2/ha}


  If ((CatSums[0,Mangrove] / 10000) / Hectares  > 0.005) THEN Tropical := TRUE
         {          ha mangrove   }  {ha overall}        ELSE Tropical := FALSE;

  For i := 0 to Site.NOutputSites + Site.MaxROS do
    Begin
      Hectares1 := 0.0;
      FOR CC := FirstCategory TO LastCategory DO
       If CC<>Blank then  {Hectares 1 excludes blank}
        Hectares1 := Hectares1 + CatSums[i,CC]  / 10000;
          {ha}          {ha}       {m2}       {m2/ha}

       RowLabel[countiter] := IntToStr(Year);
       Summary [i,countiter,0] := Site.GlobalSite.NewSL;

       For CC := FirstCategory To LastCategory DO                   // write SLAMM categories to array in HA
         Summary[i,countiter,ORD(cc)+1] := CatSums[i,cc] / 10000;
                            {ha}             {m2}          {m2/ha}

       If i=0
         then Summary[i,countiter,ORD(LastCategory)+2] := SAV_KM
         else Summary[i,countiter,ORD(LastCategory)+2] := -9999;


{      if i<=NOutputSites // no ROS FractalD or ShoreProt
          Summary[i,countiter,ORD(LastCategory)+2] :+ FRs[i].FractalD[0];
           Summary[i,countiter,ORD(LastCategory)+3] := FRs[i].FractalD[1];
           Summary[i,countiter,ORD(LastCategory)+4] := FRs[i].ShoreProtect;
         End;                                                               }

        For j := 1 to 7 do
          Begin
           Aggr := Aggregated(i,j);
           Summary[i,countiter,ORD(LastCategory)+4+j] := Aggr / 10000;
          End;
     End;

 {  Summary[Count,ORD(Blank)+3] := HaGained ;
    Summary[Count,ORD(Blank)+4] := HaLost ; }

 END; {Summarize}


Function TSLAMM_Simulation.AggName(CatNum: Integer): String;
Begin
End;

Function TSLAMM_Simulation.Aggregated(i,CatNum: Integer): Double;
Begin
  Case CatNum of
    1:  {Non Tidal} Aggregated := CatSums[i,DevDryLand] + CatSums[i,UndDryLand];
    2:  {Freshwater Non Tidal} Aggregated := CatSums[i,Swamp] + CatSums[i,CypressSwamp] + CatSums[i,InlandFreshMarsh] + CatSums[i,InlandShore];
    3:  {Open Water} Aggregated := CatSums[i,InlandOpenWater] + CatSums[i,RiverineTidal] + CatSums[i,EstuarineWater] + CatSums[i,TidalCreek] + CatSums[i,OpenOcean];
    4:  {Low Tidal } Aggregated := CatSums[i,EstuarineBeach] + CatSums[i,TidalFlat] + CatSums[i,OceanBeach] + CatSums[i,OceanFlat] + CatSums[i,RockyIntertidal] + CatSums[i,VegTidalFlat];
    5:  {Salt Marsh} Aggregated := CatSums[i,RegFloodMarsh];
    6:  {Transitional} Aggregated := CatSums[i,ScrubShrub] + CatSums[i,Mangrove] + CatSums[i,IrregFloodMarsh] + CatSums[i,BackShore];
    else {Freshwater Tidal} Aggregated := CatSums[i,TidalFreshMarsh] + CatSums[i,TidalSwamp];
  end; {case}

End;

