//SLAMM SOURCE CODE copyright (c) 2009 - 2012 Warren Pinnacle Consulting, Inc.
//Code Use and Redistribution is Subject to Licensing, SEE SLAMM_License.txt
//

{--------------------------------------------------------------------------------------------}
Function TSLAMM_Simulation.DynamicAccretion(Cell:PCompressedCell; SS: TSubsite; ModelNum: Integer): Double;  {calculated accr, mm/year}
Var AElev, DFactor, SFactor: Double;
    Elev, ElevMax, ElevMin: Double;
    Cat: Category;
    Roots: Array[1..4] of Double;
    Root3valid, Root4Valid : Boolean;
    Pctile, ShapePct,MnS,MxS: Double;

    Function Shape(SPct:Double): Double;
    Begin
      Shape := SS.AccrC[ModelNum] * (1-SPct)  + SS.AccrB[ModelNum] * SQR(1-SPct) + SS.AccrA[ModelNum] * Power(1-SPct,3);
    End;

    Function Root(Pos: Boolean): Double;  //returns min/max shape percent by finding the root of the derivative of the cubic eqn.
    Var mult: Double;
    Begin
      Result := 0.0;
      If SS.AccrA[ModelNum] = 0 then exit;  //avoid divide by zero

      If Pos then Mult :=  1.0
             else Mult := -1.0;
      Root := 1+((2*SS.AccrB[ModelNum])+Mult*SQRT(((2*SS.AccrB[ModelNum])*(2*SS.AccrB[ModelNum]))-
                (4*(3*SS.AccrA[ModelNum])*SS.AccrC[ModelNum])))/(2*(3*SS.AccrA[ModelNum]));
    End;

    Procedure CalcRoots;
    Var Rt: Double;
    Begin
      Roots[1] := Shape(1);
      Roots[2] := Shape(0);

      Rt := Root(True);
      Root3valid := (Rt>0) and (Rt < 1);
      If Root3Valid then Roots[3] := Shape(Rt);

      Rt := Root(False);
      Root4valid := (Rt>0) and (Rt < 1);
      If Root4Valid then Roots[4] := Shape(Rt);
    End;

    Function MinShape: Double;
    Begin
      MinShape := Min(Roots[1],Roots[2]);
      If Root3Valid then MinShape := Min(Result,Roots[3]);
      If Root4Valid then MinShape := Min(Result,Roots[4]);
    End;

    Function MaxShape: Double;
    Begin
      MaxShape := Max(Roots[1],Roots[2]);
      If Root3Valid then MaxShape := Max(Result,Roots[3]);
      If Root4Valid then MaxShape := Max(Result,Roots[4]);
    End;


Begin
   Case ModelNum of
     0: Cat := RegFloodMarsh;
     1: Cat := IrregFloodMarsh;
     2: Cat := TidalFlat;
     else Cat := TidalFreshMarsh;
   End; {Case}

   CalcRoots;
   MnS := MinShape;
   MxS := MaxShape;

   ElevMin := LowerBound(Cat,SS);
   ElevMax := UpperBound(Cat,SS);
   Elev := CatElev(Cell,Cat);

   If Elev<ElevMin then Elev := ElevMin;
   If Elev>ElevMax then Elev := ElevMax;
//   Elev2 := ElevMax-Elev;

   If (ElevMax-ElevMin) > 0
     then ShapePct := (Elev-ElevMin)/(ElevMax-ElevMin)
     else ShapePct := 0.5;

   With SS do
     Begin
       If MxS-MnS > 0
         then Pctile := (Shape(ShapePct)-MnS)/(MxS-MnS)
         else Pctile := 0.5;

       AElev := MinAccr[ModelNum] +( MaxAccr[ModelNum]-MinAccr[ModelNum])* Pctile;
       {Elevation Effects}

       DFactor := DMin[ModelNum];
       If (Cell.D2Channel >=0) and (DistEffectMax[ModelNum] > 0) {distance to channel has been calculated}
         then
           Begin
             If (Cell.D2Channel > DistEffectMax[ModelNum])
               then DFactor := Dmin[ModelNum]
               else DFactor := 1 - Cell.D2Channel  / DistEffectMax[ModelNum] * (1-Dmin[ModelNum]);
           End;
       {Distance to Channel}

       SFactor := SNonTMax[ModelNum];
       If (Cell.Sal[2] >= 0) and (STMaxZone[ModelNum]>0) {salinity level has been calculated}
         then
           Begin
             If  (Cell.Sal[2] > SalinityTMax[ModelNum] - (STmaxZone[ModelNum]*0.5))
             and (Cell.Sal[2] < SalinityTMax[ModelNum] + (STmaxZone[ModelNum]*0.5))
               then	SFactor := SNonTMax[ModelNum]+(1-SNonTMax[ModelNum]) *
                               (1- ABS(SalinityTMax[ModelNum]-Cell.Sal[2]) / (STMaxZone[ModelNum]*0.5))
               else SFactor := SNonTMax[ModelNum];
           End;
       {Salinity Turbidity Max Factor}

     End;  {with SS}

   Result := AElev * DFactor * SFactor;
End;





Function TSLAMM_Simulation.RegFloodAccrete(Cell:PCompressedCell; SS: TSubsite): Double;   {accretion rates, mm / year}
Begin
  If SS.UseAccrModel[0] then Result := DynamicAccretion(Cell,SS,0)
                        else Result := SS.FixedRegFloodAccr;

End;

Function TSLAMM_Simulation.IrregFloodAccrete(Cell:PCompressedCell; SS: TSubsite): Double; {accretion rates, mm / year}
Begin
  If SS.UseAccrModel[1] then Result := DynamicAccretion(Cell,SS,1)
                        else Result := SS.FixedIrregFloodAccr;

End;

Function TSLAMM_Simulation.TidalFlat_Sed(Cell:PCompressedCell; SS: TSubsite): Double;  {accretion rates, mm / year}
Begin
  If SS.UseAccrModel[2] then Result := DynamicAccretion(Cell,SS,2)   //1/23/2011 Not using Veg. Tidal Flat Accr different than T.Flat
                        else Result := SS.Fixed_TF_Beach_Sed;

End;

Function TSLAMM_Simulation.TidalFreshAccrete(Cell:PCompressedCell; SS: TSubsite): Double;   {accretion rates, mm / year, Use Veg. Tidal Flat Model Number 1/23/2011}
Begin
  If SS.UseAccrModel[3] then Result := DynamicAccretion(Cell,SS,3)
                        else Result := SS.FixedTideFreshAccr;
End;

{--------------------------------------------------------------------------------------------}

Procedure TSLAMM_Simulation.EustaticSLChange(VAR Site: TSite; VAR Year: Integer; IsT0: Boolean);      // EUSTATIC CALC

Var IPCCYearIndex: 1..4;
    DT: Integer;

        {--------------------------------------------------------------------------------}
        Function ReturnNorm(Year:Integer): Double;
        Const IPCCResults : Array[Est_Min..Est_Max,1..4,Scen_A1B..Scen_B2] of Double =
              (((28    ,27.5  ,30    ,26    ,27     ,28.5 ),  {MIN,  2025, Scen}
                (63    ,66    ,64    ,58    ,52     ,56   ),  {MIN,  2050, Scen}
                (100   ,125   ,94    ,103   ,76     ,85   ),  {MIN,  2075, Scen}
                (129   ,182   ,111   ,155   ,92     ,114  )), {MIN,  2100, Scen}
                ((76   ,81.5  ,75.5  ,74.5  ,75.5   ,79   ),  {MEAN, 2025, Scen}
                (167   ,175   ,172   ,157   ,150    ,160  ),  {MEAN, 2050, Scen}
                (278.5 ,278   ,323   ,277   ,232.5  ,255  ),  {MEAN, 2075, Scen}
                (387   ,367   ,491   ,424   ,310    ,358  )), {MEAN, 2100, Scen}
                ((128  ,128.5 ,137   ,126.5 ,128    ,134  ),  {MAX,  2025, Scen}
                (284   ,291   ,299   ,269   ,259    ,277  ),  {MAX,  2050, Scen}
                (484.5 ,553   ,491   ,478   ,412.5  ,451  ),  {MAX,  2075, Scen}
                (694   ,859   ,671   ,743   ,567    ,646  )));{MAX,  2100, Scen}

         Const FixedResults : Array [1..4,1..3] of Double =
                ((184.4, 276.7,  368.9),  {2025, Scen}
                 (409.2, 613.8,  818.4),  {2050, Scen}
                 (698.1, 1047.2, 1396.3), {2075, Scen}
                 (1000, 1500, 2000.0));   {2100, Scen}
              {--------------------------------------------------------------------}
              Function Norm_Data_Year(Year:Integer):Double;
              Var Nrm: Double;
              Begin
                If Year > 2100 then Raise ESLAMMERROR.Create('SLAMM Predictions do not extend beyond 2100.');
                Case Year of
                  2050: IPCCYearIndex := 2;
                  2075: IPCCYearIndex := 3;
                  2100: IPCCYearIndex := 4;
                  else  IPCCYearIndex := 1;
                End; {Case}

                If Running_Fixed
                  then  Nrm := FixedResults[IPCCYearIndex,FixedScen] * 0.1
                  else  Nrm := IPCCResults[IPCCSLEst,IPCCYearIndex,IPCCSLRate] * 0.1;
                        {cm}        {mm}                               {mm/cm}

                Norm_Data_Year := Nrm;

              End; {Norm_Data_Year}
              {--------------------------------------------------------------------}

        Var UpperYear, LowerYear: Integer;
            UpperNorm, LowerNorm, ScenBy2100: Double;

        Begin {ReturnNorm}
          If Running_Custom then
            Begin
              Running_Fixed := False;
              IPCCSLRate := Scen_A1B;
              IPCCSLEst := Est_Max; {scale up custom from A1B-Max scenario}
            End;

          If (Year>2000) and ((Year Mod 25) = 0)
            then ReturnNorm := Norm_Data_Year(Year)
            else Begin
                   LowerYear := (Year Div 25) * 25;
                   UpperYear := (Year Div 25) * 25 + 25;

                   If LowerYear > 2000 then LowerNorm := Norm_Data_Year(LowerYear)
                     else Begin
                            LowerYear := 1990;  {linearly interpolate to zero at start date}
                            LowerNorm := 0;
                          End;
                   If (UpperYear < 2025) then UpperYear := 2025;
                   UpperNorm := Norm_Data_Year(UpperYear);
                   ReturnNorm := LinearInterpolate(LowerNorm,UpperNorm,LowerYear,UpperYear,Year);
                 End;

           If (Running_Custom) then
            Begin
              If CustomSLR<0 then CustomSLR := 0;
              ScenBy2100 := Norm_Data_Year(2100);   // First Get A1B-Max Prediction for 2100
              Result := Result * (CustomSLR * 100) / ScenBy2100;  {i.e. if SLR by 2100 is 50% Scen by 2100 then all dates are scaled by 50%}
               {cm}      {cm}       {m}       {cm}     {cm}
            End;
        End;  {ReturnNorm}
        {--------------------------------------------------------------------------------}
        Procedure GetNextYearFromString;
        Var Number, MinNum, MaxNum: Integer;
            Holder, InputNums: String;
            EOFFound : Boolean;
        Begin
          MinNum := 99999;  MaxNum := -99999;
          InputNums := YearsString;
          EOFFound:=False;

          Repeat
            Holder := AbbrString(InputNums,',');
              Try Number := StrToInt(Holder);
              Except Raise ESLAMMError.Create('Invalid Numeric Input in Specific Years String "'+YearsString+'"   Must be comma separated integers.');
              End;
            If Number > MaxNum then MaxNum := Number;
            If (Number > Year) and (Number < MinNum) then MinNum := Number;

            If Pos(',',InputNums)= 0 then EofFound:=True
                                     else Delete(InputNums,1,Pos(',',InputNums));
            If Trim(InputNums)='' then EofFound := True;
          Until EofFound;

          If MinNum = 99999 then MinNum := Year;

          DT := MinNum - Year;
          Year := MinNum;
          MaxYear := MaxNum;

          If (Year>2100) then Begin DT := DT - (Year-2100); Year := 2100; End;
        End;
        {--------------------------------------------------------------------------------}

        Procedure CalcDT; {calculate Delta T when the model is run beyond T0}
        Begin
          If Runspecificyears
            then GetNextYearFromString
            else
              Begin
                If Year < 2025
                  then DT := 2025 - Year
                  else DT := TimeStep;

                If TimeStep < 25 then
                  If Year < 2020
                    then DT := 2020 - Year
                    else DT := TimeStep;

                If TimeStep < 15 then
                  If Year < 2010
                    then DT := 2010 - Year
                    else DT := TimeStep;

                Year := Year + DT;

                If (Year>2100) then Begin DT := DT - (Year-2100); Year := 2100; End;
              End;

        End;
        {--------------------------------------------------------------------------------}

Var i: Integer;
    TSS: TSubSite;

Begin
  If IsT0 then DT := 0
          Else CalcDT;

  For i:=0 to Site.NSubSites do
    Begin
      If i=0 then TSS := Site.GlobalSite
             else TSS := Site.SubSites[i-1];
      With TSS do
        Begin
          OldSL := NewSL;
          If Not IsT0 then DeltaT := DT
                      else DeltaT := Year - NWI_Photo_Date;

          OverWashes_This_dT := 0;
          If Freq_BigStorm > 0 then
            If Year - LastOverWash > Freq_BigStorm
              then
                Begin
                  Overwashes_This_dT := Trunc((Year-LastOverwash) / Freq_BigStorm);
                  LastOverWash := LastOverWash + Trunc(Overwashes_This_dT * Freq_BigStorm);
                End;

          If Year = NWI_Photo_Date
           then Norm := 0
           else
             Begin
               If Year > 1990 then
                 Begin
                   Norm := ReturnNorm(Year);
                   {IPCC predictions start date is 1990, so adjust predictions to the NWI date which serves as the start of our simulation}
                   If (NWI_Photo_Date < 1990)
                    then Norm := Norm + (1990 - NWI_Photo_Date)   * 1.7   *  0.1     {4/28/10 JSC, made this eustatic}
                         { cm      cm    (   yr                )   mm/yr   cm/mm }
                   else if (NWI_Photo_Date > 1990)
                    then Norm := Norm - ReturnNorm(NWI_Photo_Date);
                        { cm      cm  - cm }
                 End
               Else {Year <= 1990}  {Uses Subsite Historic SLR For SLR Pre 1990}
                 Begin
                   Norm := (Year - NWI_Photo_Date) * 1.7   * 0.1          {4/28/10 JSC, made this eustatic}
                  { cm     (   yr                )   mm/yr  cm/mm }
                 End;
              End;


          NewSL := Norm *0.01;  {NewSL is Eustatic SLR since simulation start}
          { m       cm   }

          SLRise := NewSL - OldSL;   {Eustatic SLR this time-step}
           {m}       {m}     {m}

        End;
    End;
End;

Function TSLAMM_Simulation.UpdateElevations: Boolean;  {update cell elevations for entire map based on SLR, uplift, subsidence, accretion}
Var ER, EC, i, fw, nfw: Integer;
    ECl: CompressedCell;
    Subsite: TSubsite;
    ThisCat: Category;
    Accretion, ThisElev, TurbFactor: Double;
    CellSet: Boolean;

Begin
 Result := True;
 ProgForm.ProgressLabel.Caption:='Updating Elevations';

 FOR ER := 0 To (Site.Rows-1) Do
  FOR EC := 0 To (Site.Cols-1) Do
   Begin
     If EC=1 then
       begin
         Result := ProgForm.Update2Gages(Trunc((Er)/Site.Rows*100));
         If Not Result then Exit;
       end;

    RetA(ER, EC, ECl);
    Subsite := Site.GetSubSite(EC {x},ER {y},@ECl);

    CellSet:= False;

    For i := 1 to NUM_CAT_COMPRESS do
      Begin
        ThisCat := ECl.Cats[i];
        If ThisCat <> Blank then
          Begin
            CellSet := True;
            ThisElev := ECL.MinElevs[i] - SubSite.SLRise;     {MTL has moved so all elevs relative to MTL have changed.  Adjust for eustatic changes first}
                             {m}          {m per Delta T}

            with Subsite do
              ThisElev := ThisElev + ECl.Uplift * DeltaT * 0.01;   {account for uplift or subsidence}
               {m}         {m}          {cm/yr}    {yr}   {cm/m}

            Accretion := 0;
            If (Not IncludeDikes) or (Not ECl.ProtDikes) then with Subsite do      {calculate cell-specific accretion for non diked cells}
              Case ThisCat of
                RegFloodMarsh     : Accretion := DeltaT * RegFloodAccrete(@ECl,SubSite)     * 0.001;
                ScrubShrub,
                  IrregFloodMarsh : Accretion := DeltaT * IrregFloodAccrete(@ECl,SubSite)   * 0.001;
                VegTidalFLat      : Accretion := DeltaT * TidalFlat_Sed(@ECl,SubSite)  * 0.001;
                TidalFlat,
                  OceanFlat       : Accretion := DeltaT * TidalFlat_Sed(@ECl,SubSite) * 0.001;
                EstuarineBeach,
                  OceanBeach      : Accretion := DeltaT * Fixed_TF_Beach_Sed * 0.001;
                TidalSwamp        : Accretion := DeltaT * TSwampAccr         * 0.001;
                Swamp,
                  CypressSwamp    : Accretion := DeltaT * SwampAccr          * 0.001;
                Mangrove          : Accretion := DeltaT * MangroveAccr       * 0.001; 
                TidalFreshMarsh   : Accretion := DeltaT * TidalFreshAccrete(@ECl,SubSite) * 0.001;  //1/21/2011
                InlandFreshMarsh: Accretion   := DeltaT * InlandFreshAccr * 0.001;
                                       {m}         {yr}        {mm/yr}        {mm/m}
              End; {Case}

            {Add Turbidity Factor}
            TurbFactor := 1.0;
            nfw := 0;
            If Accretion > 0 then
              For fw := 0 to NumFWFlows -1 do
               If FWInfluenced(ER,EC,fw) then
                 If FWFlows[fw].UseTurbidities then
                   Begin
                     inc(nfw);
                     If nfw=1 then TurbFactor :=  FWFlows[fw].TurbidityByYear(Year)
                              else TurbFactor := (FWFlows[fw].TurbidityByYear(Year) + (TurbFactor * (nfw-1))) / nfw   // calcuate average turbfactor
                   End;

            ThisElev := ThisElev + Accretion* TurbFactor;          {account for accretion}
            ECL.Minelevs[i] := ThisElev;
          End; {if}
      End; {for i}

    If CellSet then SetA(ER,EC,ECL)
   End; {For ER, EC}
End;


{--------------------------------------------------------------------------------------------}

{-------------------------------------------}
{      Transfer percentages of Classes      }
{         R. A. Park    2/21-29/88          }
{        JSC Modified,  1998, 2005-2007     }
{-------------------------------------------}

PROCEDURE TSLAMM_Simulation.TRANSFER(Cell: PCompressedCell;
                                     EachRow,EachCol: Integer; ErodeStep: Boolean);
Var CC: Category;
    Diked: Boolean;
VAR
  WetlandType: Category;  {Set in TestNear}
  WaterTable : double;    {Set in TestNear}
  DistancetoWater: double;       {Set in TestNear}
  AdjMaxFetch: Double;
  NearSalt,
  NearWater:   Boolean;
  ERODING      : Boolean;
  CellFWInfluenced: Boolean;
  SubSite      : TSubSite;

    {--------------------------------------------------------------------}
    Function AdjCell(Var AdjRow,AdjCol: Integer; Lee: Boolean): Boolean;

    {********************************************}
    {* Return adjacent cell to lee if Lee=True  *}
    {* AdjRow and AdjCol are inputs and outputs *}
    {* Function False if out-of-bounds          *}
    {* JClough, December 2005 (adds modularity) *}
    {********************************************}
    Var Step: Integer;
    Begin
      If Lee then Step := -1
             else Step :=  1;

      Case SubSite.Direction_OffShore of
         Westerly : AdjCol := AdjCol - Step;
         Easterly : AdjCol := AdjCol + Step;
         Northerly: AdjRow := AdjRow - Step;
         Southerly: AdjRow := AdjRow + Step;
        End; {Case}

      AdjCell := (AdjRow>=0) and (AdjCol>=0) and
                 (AdjRow<Site.Rows) and (AdjCol<Site.Cols);
    End;
    {--------------------------------------------------------------------}


    {*******************************************}
    {            convert Classes                }
    {         R. A. Park    2/21/88             }
    {*******************************************}
    Procedure Convert(Cell: PCompressedCell; FromCat, ToCat: Category; LowBound: Single);
    {converts a cell from category to category based on FracLost[FromCategory]}
    Var Transferred: double;
        FromMinElev: Single;
    Begin
      If FracLost[FromCat] = -9999
        then Raise ESlammError.Create('FRACLOST Initialization Problem, '+TitleCat[FromCat]);

     With Cell^ DO
      Begin
        If FracLost[FromCat] > 1.0 Then FracLost[FromCat] := 1.0;
        If FracLost[FromCat] < 0.0 Then FracLost[FromCat] := 0.0;

        If CellWidth(Cell,FromCat) < 0.001
          Then Transferred := 0.0  {to avoid "pass through" of small amts.}
          Else Transferred := FracLost[FromCat] * CellWidth(Cell,FromCat);

        IF Transferred > 0.0 Then
          Begin
            FromMinElev := CatElev(Cell,FromCat);
            SetCellWidth(Cell,FromCat,CellWidth(Cell,FromCat) - Transferred);  {loss first to minimize data loss due to compression}
            SetCellWidth(Cell,ToCat,  CellWidth(Cell,ToCat)   + Transferred);

            If (CatElev(Cell,ToCat) >= 999)
              then  SetCatElev(Cell,ToCat,FromMinElev)
              else if (CatElev(Cell,ToCat) > FromMinElev)
                then  SetCatElev(Cell,ToCat,FromMinElev);  {Set the elevation of the category being transferred to if it did not exist or was errant}

            If LowBound <> -99 then
              if CellWidth(Cell,ToCat) > 0  {If "to cat" wasn't set because it is too small, don't nudge up "from cat"}
                 then SetCatElev(Cell,FromCat,LowBound);
          End; {Trans > 0}
      End; {With Cell}
    End; {Convert}

{********************************************}
{       Calculate Frac Lost using LowBound,  }
{             Minelev and Cell Slope         }
{         J Clough  December 2005            }
{********************************************}
Procedure CalcFracLostBySlope(Cell: PCompressedCell; Cat: Category; LowBound: Double);
Var Wid: Single;
    MinElev: Single;
Begin
 FracLost[Cat] := 0;
 If ERODING Then Exit;

 MinElev := CatElev(Cell,Cat);
 if (MinElev>=LowBound) then Exit;  //JSC 1/10/2012

 Wid := CellWidth(Cell,Cat);
 If Wid < Tiny then Exit;   {avoid potential divide by zero}

 With Cell^ do
  If TanSlope = 0 then FracLost[Cat] := 1
     else If TanSlope < 0 then FracLost[Cat] := 0
     else FracLost[Cat] := ((LowBound - MinElev) / TanSlope) / Wid;
  {frac. of cat to conv.}      {m}        {m}     {unitless}    {m}

  If FracLost[Cat] < 0 then FracLost[Cat] := 0; {2-22-06}
End;

{*******************************************}
{         represent migration of            }
{         water table near shoreline        }
{         R. A. Park    9/27/91             }
{         J.Clough December 2005            }
{*******************************************}
Procedure Saturate(Cell: PCompressedCell; FromCat,ToCat: Category);
Begin
    IF ERODING Then Exit;
    CalcFracLostBySlope(Cell,FromCat,WaterTable);
    Convert(Cell,FromCat,ToCat,WaterTable);
End; {Saturate}




{*******************************************}
{         Incorporate Salinity in Wetland   }
{         fate                              }
{         J.Clough November 2009            }
{*******************************************}

Procedure ConvertCellbySalinity(Cell: PCompressedCell; SRs: TSalinityRules);
Var Cat: Category;
    nrules, i, j, convnum: Integer;
    TSR: TSalinityRule;
    CellSal: Double;
    //NumSalMetrics: integer;
Begin
 //If Cell.Sal[NUM_SAL_METRICS] < -1 then exit;   //not initialized for salinity, probably "extent only"

 FOR i := 1 to NUM_CAT_COMPRESS DO
   If (Cell.Widths[i] > 0) and
      (Cell.Cats[i] <> Blank) then
     Begin
       Cat := Cell.Cats[i];
       nrules := SRs.CountRules(Cat);
       for j  := 1 to nrules do
         Begin
           TSR := SRs.FindRule(Cat,j);    // Handle case of multiple salinity rules
           If TSR <> nil then
             Begin
               // If salinity of the 1st salinity metrics is not null but the one of 4th metrics is null then there must be a raster
               if Cell.Sal[NUM_SAL_METRICS]<-1 then
                TSR.SalinityTide := 1;

               // Get the salinity value
               CellSal := Cell.Sal[TSR.SalinityTide];

               For convnum:= 1 to 2 do
                if TSR <> nil then
               // Apply the conversion rule, go through two steps if required
                 if (TSR.GreaterThan and (CellSal > TSR.SalinityLevel)) or
                    ((Not TSR.GreaterThan) and (CellSal < TSR.SalinityLevel)) then
                   begin
                     FracLost[Cat] := 1.0;
                     Convert(Cell,Cat,TSR.ToCat,-99);
                     Cat := TSR.ToCat;                     // Allow for second category change
                     TSR := SRs.FindRule(TSR.ToCat,1);     // Allow for second category change
                   end;
             End; {TSR <>nil}
         End;
     End;
End; {procedure}


PROCEDURE Inundate(Cell: PCompressedCell; FromCat: Category);  {inundation of dry land}

    {--------------------------------------------------------------------}
    Function OceanNearer: Boolean;  {which is nearer, ocean or estuary?}
    Var i: Integer;
        LeeR,LeeC: Integer;
        C2: CompressedCell;
    Begin
      OceanNearer := True;
      LeeR := EachRow; LeeC := EachCol;
      For i := 0 to Trunc(DistancetoWater/Site.Scale)+1 do
        If AdjCell(LeeR,LeeC,True) then
          Begin
            RetA(LeeR,LeeC,C2);
            If CellWidth(@C2,EstuarineWater) > 0 then OceanNearer := False;
          End;
    End;
    {--------------------------------------------------------------------}

BEGIN
 IF ERODING Then Exit;

 IF not ProtectAll Then
  Begin
    With Cell^ DO
     If FALSE {(CatSums[RockyIntertidal] > CatSums[RegFloodMarsh]) {Rocky Intertidal Assumption Removed}
      Then Convert(Cell,FromCat,RockyIntertidal,-99)
      Else
       Begin
        If (AdjOcean) and ((not IncludeDikes) or (not Cell.ProtDikes))
          Then Begin
                 If OceanNearer then Convert(Cell,FromCat,OceanBeach,LowerBound(FromCat,Subsite))
                                else Convert(Cell,FromCat,EstuarineBeach,LowerBound(FromCat,Subsite))

               End
          Else
           If (AdjWater and (Erosion > Heavy)) and ((not IncludeDikes) or (not Cell.ProtDikes))
             Then Convert(Cell, FromCat, EstuarineBeach,LowerBound(FromCat,Subsite))
             Else
               Begin
                 IF (Tropical AND NearWater)
                   Then Convert(Cell,FromCat,Mangrove,LowerBound(FromCat,Subsite))
                   else If (FromCat <> Backshore)
                     then
                       Begin
                         if CellFwInfluenced then Convert(Cell,FromCat,tidalswamp,LowerBound(FromCat,Subsite))   //2-13-2012
                                             else Convert(Cell,FromCat,ScrubShrub,LowerBound(FromCat,Subsite));
                       End
                     else Convert(Cell,FromCat,EstuarineBeach,LowerBound(FromCat,Subsite));
               End; {else not exposed and erosion not heavy}
       End; {not rocky}
  End; {not ProtAll}
END; {INUNDATE}

{*******************************************}
{      erode beach using Bruun observation  }
{          R. A. Park    4/4/88             }
{       Reworked JClough December 2005      }
{ Recession = 100 * SLRise (Bruun '62, '86) }
{*******************************************}
Procedure ErodeBeach(Cell: PCompressedCell);
Var Recession: double;
    ErosionThisCell: Double;
    LocalSLRise: Double;
    BeachWidth: Double;
Begin
  If Not ERODING then Exit;  {don't calculate erosion during inundation time-step}
  If Not UseBruun then Exit;

//   IF CellWidth(Cell,OpenOcean) > 0 then DistancetoWater := 0; {cell contains ocean}

  LocalSLRise := SubSite.SLRise - Cell.Uplift * SubSite.DeltaT * 0.01;
    {m}                  {m}           {cm/yr}          {yr}     {cm/m}

  Recession := 100.0 * LocalSLRise;

  ErosionThisCell := Recession - DistancetoWater;
     {m}                {m}       {m from front edge to open ocean}

  If ErosionThisCell <= 0 then Exit;

  BeachWidth := CellWidth(Cell,OceanBeach);
  If ErosionThisCell > BeachWidth then ErosionThisCell := BeachWidth;
       {m}                 {m}

  FracLost[OceanBeach] := ErosionThisCell/BeachWidth;
  Convert(Cell,OceanBeach,OpenOcean,-99);
  Cell.BTFErosionLoss := Cell.BTFErosionLoss + ErosionThisCell;

End;


(********************************)
(* obtain qual. est. of erosion *)
(********************************)
Procedure Erode(Cell: PCompressedCell; Var Erosion: ErosionScale);

BEGIN
 With Cell^ DO
  Begin
   IF (AdjMaxFetch <= 1) THEN Erosion := None;
   IF (AdjMaxFetch > 1)  AND (AdjMaxFetch <= 3) {km}  THEN Erosion := Little;
   IF (AdjMaxFetch > 3)  AND (AdjMaxFetch <= 9)  THEN Erosion := Moderate;
   IF (AdjMaxFetch > 9)  AND (AdjMaxFetch <= 20) THEN Erosion := Heavy;
   IF (AdjMaxFetch > 20) THEN Erosion := Severe;
  End; {with}
END; {Erosion}

{********************************************}
{  Take into account width of category being }
{  Eroded,  JClough December 2005            }
{*******************************************}
Function Frac_To_Erode(Cell: PCompressedCell; ErodePerYear: Double; Clss:Category): Double;
Var TotErosion: Double;
    WidthErodedOS: Double; {Width Eroded in Off Shore direction}
    AdjRow,AdjCol: Integer;
    EdgeFound: Boolean;
    CatWidth, ErosionLossMeters: Double;

    Procedure AddToWidth;
    Var AC: CompressedCell; {Adjacent Offshore Cell}
    Begin
       RetA(AdjRow,AdjCol,AC);

       If CellWidth(@AC,Clss)>0
         then WidthErodedOS := WidthErodedOS + CellWidth(@AC,Clss)
         else EdgeFound := True;

       If EdgeFound and not(GetBit(MostlyWater,AdjRow,AdjCol)) then WidthErodedOS := 1e6;  {not adjacent to water, no erosion, 1/13/2010}
       WidthErodedOS := WidthErodedOS + ErodeMatrix[Site.Cols*(AdjRow)+(AdjCol)];
    End;

Begin
  Frac_To_Erode := 0;
  IF NOT ERODING Then Exit;
  If ErodePerYear <=0 then exit;
  If SubSite.DeltaT<=0 then exit;

  TotErosion := ErodePerYear * SubSite.DeltaT;  {total horizontal erosion in time-step}
  {m / dT}        {m / yr}             {y / dt}

  WidthErodedOS := 0;

  AdjRow := EachRow;
  AdjCol := EachCol;
  EdgeFound := False;
  Repeat
    If AdjCell(AdjRow,AdjCol,False) then AddToWidth    {look off-shore and add width}
                                    else EdgeFound := True;  {off the map}
    If WidthErodedOS > TotErosion then Exit;           {No erosion for this cell, enough erosion took place on off-shore side of cell}
  Until EdgeFound;

  CatWidth := CellWidth(Cell,Clss);
  Frac_To_Erode  := (TotErosion - WidthErodedOS)/ CatWidth;
   {frac / dT}          {m/dT}            {m/dT}      {m}

  If Result > 1 then ErosionLossMeters := CatWidth
                else ErosionLossMeters := TotErosion - WidthErodedOS;

  If Clss in [OceanBeach,EstuarineBeach,TidalFlat]
    then Cell.BTFErosionLoss := Cell.BTFErosionLoss + ErosionLossMeters
    else Cell.ErosionLoss := Cell.ErosionLoss + ErosionLossMeters;

  If Trunc(TotErosion-WidthErodedOS) > ErodeMatrix[Site.Cols*(EachRow)+(EachCol)] then        {Keep track of the erosion at this cell in this time-step thus far}
    ErodeMatrix[Site.Cols*(EachRow)+(EachCol)] := Trunc(TotErosion-WidthErodedOS);
End;

{*******************************************}
{       compute erosion of marsh            }
{         R. A. Park    3/4/88              }
{*******************************************}
Procedure ErodeMarsh(Cell: PCompressedCell; Clss: Category);
Begin
  IF not ERODING then Exit;
  FracLost[Clss] := Frac_To_Erode(Cell,SubSite.MarshErosion,Clss);
  Convert(Cell,Clss,Tidalflat,-99);   {erosion will always lead to tidal flats}
End;

{********************************************}
{       compute erosion of tidal flat        }
{         R. A. Park    1/26/99              }
{********************************************}
Procedure ErodeTFlat(Cell: PCompressedCell; Clss: Category; Connected_to_Water: Boolean);
Begin
  IF not ERODING then Exit;
  FracLost[Clss] := Frac_To_Erode(Cell, SubSite.TFlatErosion,Clss);
  If Clss=OceanBeach then Convert(Cell,Clss,OpenOcean,-99)
                     else If Connected_to_Water then Convert(Cell,Clss,EstuarineWater,-99)
                                        else Convert(Cell,Clss,InlandOpenWater,-99);

End;

{*******************************************}
{         compute erosion of swamp          }
{         R. A. Park    3/17/88             }
{*******************************************}
Procedure ErodeSwamp(Cell: PCompressedCell; Clss: Category);
Begin
  IF not ERODING then Exit;
  FracLost[Clss] := Frac_To_Erode(Cell, SubSite.SwampErosion,Clss);
  Convert(Cell,Clss,Tidalflat,-99);
End;

(********************************)
(* account for lack of sedimen- *)
(* tation and decreasing eleva- *)
(* tion in diked wetlands with  *)
(* failure of dike if too low   *)
(*      R. A. Park  4/12/88     *)
(********************************)
Procedure DikedWetland(Cell: PCompressedCell; Clss: Category);

    Procedure SetSalinityByPumpCell;
    Var PumpRow,PumpCol: Integer;
        PumpCell: CompressedCell;
    Begin
      If Cell.DikeIndex< 0 then exit;  {in case pump cell code disabled}
      PumpRow := DikeInfo[Cell.DikeIndex-1].UpRow;
      PumpCol := DikeInfo[Cell.DikeIndex-1].UpCol;
      RetA(PumpRow,PumpCol,PumpCell);
      Cell.Sal := PumpCell.Sal;
    End;

Var
  DikeElev     : double;
Begin
  IF ERODING Then Exit;
  If not ClassicDike Then Exit;  // 5/23/2012 the case of classic dikes behind connectivity dikes was causing problems.  Due to classic dike conversion, open water was occurring
                                 // which caused connectivity to occur even when it should not.  Therefore, classic dike 2M conversion is disabled when height-sensitive dikes are included in a simulation.

  If (CellFwInfluenced) and (Clss in [RegFloodMarsh,TidalFreshMarsh,IrregFloodMarsh,TidalSwamp]) then
      SetSalinityByPumpCell;

  DikeElev := CatElev(Cell,Clss);
  IF DikeElev < -2.0 Then
    Begin
      FracLost[Clss] := 1.0;
      If AdjOcean Then Convert(Cell,Clss,OpenOcean,-99)        {failure of dike}
                  Else Convert(Cell,Clss,EstuarineWater,-99);  {failure of dike}

    End;
End;

{*******************************************}
{   Simulate washover of drowned barrier    }
{         R. A. Park    3/3/88              }
{  Rewritten Dec 2005, JSC, Account for     }
{   30 m cell size, 500 m effect            }
{  Modified 9-2008 JSC, Allow multi o-washes}
{  Each time-step and variable effects      }
{*******************************************}
Procedure TestOverwash(Cell: PCompressedCell);
Var WSalt, WTrans, WMang, WBrack: Double;

   {-------------------------------------------------------------------------}
    Procedure Overwash;
    {March Specified distance (500m) to lee and simulate effects of Overwash,  JSC Dec 2005}
    Var WCDryLand,WCEstuary,WCBeach,WCMang,WCTrans,WCSalt: Double;  {Width Converted for these classes}

        {------------------------------------------------}
        Procedure WashCell(Row, Col: Integer);
        Var AC: CompressedCell;

              Function FracLostByWidth (ThisWidth: Double; Var Converted: Double; MaxWidth: Double): Double;
              Begin
                FracLostByWidth := 0.0;
                IF (ThisWidth>0) and (Converted < MaxWidth) then
                  Begin
                    Converted := Converted + ThisWidth;
                    If Converted < MaxWidth
                      then FracLostByWidth := 1.0
                      else FracLostByWidth := 1 - (Converted - MaxWidth)/ThisWidth;
                  End;
              End;

        Begin
          RetA(Row,Col,AC);
          With AC do with subsite do
            Begin
              FracLost[RegFloodMarsh] := FracLostByWidth(CellWidth(@AC,RegFloodMarsh),WCSalt,WSalt*OW_Marsh_Pct_Loss*0.01);     {Loss of 50% over 500m}
              Convert(@AC, RegFloodMarsh, EstuarineBeach,-99);

              FracLost[ScrubShrub] := FracLostByWidth(CellWidth(@AC,ScrubShrub),WCTrans,WTrans*OW_Marsh_Pct_Loss*0.01);  {Loss of 50% over 500m}
              Convert(@AC, ScrubShrub, UndDryLand,-99);  {SLAMM 5 increase min elevation}

              FracLost[IrregFloodMarsh] := FracLostByWidth(CellWidth(@AC,IrregFloodMarsh),WCTrans,WBrack*OW_Marsh_Pct_Loss*0.01);  {Loss of 50% over 500m}
              Convert(@AC, IrregFloodMarsh, UndDryLand,-99);  {SLAMM 5 increase min elevation}

              FracLost[Mangrove] := FracLostByWidth(CellWidth(@AC,Mangrove),WCMang,WMang*OW_Mang_Pct_Loss*0.01);  {Loss of 25% over 500m, Professional Judgement}
              Convert(@AC, Mangrove, EstuarineBeach,-99);

              FracLost[OceanBeach] := FracLostByWidth(CellWidth(@AC,OceanBeach),WCBeach,OW_Beach_to_Ocean);
              Convert(@AC, OceanBeach, OpenOcean,-99); {Ocean Beach moves 30 meters back or user spec. amount}

              FracLost[EstuarineWater] := FracLostByWidth(CellWidth(@AC,EstuarineWater),WCEstuary,OW_Est_to_Beach);
              Convert(@AC,EstuarineWater,RegFloodMarsh,-99);

              FracLost[UndDryLand] := FracLostByWidth(CellWidth(@AC,UndDryLand),WCDryLand,OW_Dryland_to_Beach);
              Convert(@AC,UndDryLand,OceanBeach,-99);

              FracLost[DevDryLand] := FracLostByWidth(CellWidth(@AC,DevDryLand),WCDryLand,OW_Dryland_to_Beach);
              Convert(@AC,DevDryLand,OceanBeach,-99);

              SetA(Row,Col,AC);

              If Display_Screen_Maps then  {Update Screen too}
                If Gridform.IsVisible then
                  Gridform.DrawCell(@AC,Row,Col,GetCellCat(@AC),False,False,True,0);
            End; {With AC}
        End; {WashCell}
        {------------------------------------------------}

    Var AdjRow,AdjCol: Integer;
        I, J: Integer;
    Begin  {overwash}
        WCDryLand:=0; WCEstuary:=0; WCBeach:=0; WCMang:=0; WCTrans:=0; WCSalt:=0;
        J := Round(subsite.OW_Max_Width_Overwash/Site.Scale);
        AdjRow := EachRow; AdjCol := EachCol;
        WashCell(AdjRow,AdjCol);
        For I := 1 to J Do   {march specified m, usually 500 adjacent to lee}
          If AdjCell(AdjRow,AdjCol,True) then WashCell(AdjRow,AdjCol);
    End; {Overwash}

    {------------------------------------------------------------------------------------------------}
    Function CellIsFrontEdge: Boolean; {find the very front edge of the ocean beach}
    Var AC: CompressedCell; {adjacent cell}
        AdjRow, AdjCol: Integer;
    Begin
      CellIsFrontEdge := False;
      If (CellWidth(Cell,OceanBeach)>0.0) {must be beach}
       Then
        Begin
         AdjRow := EachRow; AdjCol := EachCol; {set for adjcell use}
         If CellWidth(Cell,OpenOcean)>0.0  {ocean in this cell too, this must be the front edge}
          then CellIsFrontEdge := True
          else
           If AdjCell(AdjRow,AdjCol,False) then
             Begin
              RetA(AdjRow,AdjCol,AC);
              CellIsFrontEdge := (CellWidth(@AC,OceanBeach) = 0.0) and (CellWidth(@AC,OpenOcean) > 0.5)
                    {There's no beach next cell over but there is ocean, so current cell is the front edge}
             End;
        End;
    End;
    {------------------------------------------------------------------------------------------------}
    Function WaterToLee: Boolean;
    {Is there estuarine water or RegFloodMarsh / mangrove/ estuarine beach 500 meters (or spec distance) to lee of beach front?}
    {If there's Rocky Intertidal there, no Overwash is assumed to occur}
    Var  WTL, RockyFound: Boolean;

        {------------------------------------------------}
        Procedure Adj(Row, Col, I: Integer);
        Var AC: CompressedCell; {adjacent cell}
        Begin
          RetA(Row,Col,AC);
          If CellWidth(@AC,RockyInterTidal) > 0.1 then RockyFound := True;
          If Not WTL then
            If CellWidth(@AC,EstuarineWater) + CellWidth(@AC,ScrubShrub) + CellWidth(@AC,IrregFloodMarsh) +
               CellWidth(@AC,Mangrove) + CellWidth(@AC,EstuarineBeach) +  CellWidth(@AC,RegFloodMarsh) > 0.3
                 then WTL := True;
          WSalt  := WSalt + CellWidth(@AC,RegFloodMarsh);
          WMang  := WMang + CellWidth(@AC,Mangrove);
          WTrans := WTrans + CellWidth(@AC,ScrubShrub);
          WBrack := WBrack + CellWidth(@AC,IrregFloodMarsh);
        End; {Adj}
        {------------------------------------------------}

     Var AdjRow,AdjCol: Integer;
         I, J: Integer;
     BEGIN
        WTL := False; RockyFound := False;
        WSalt := 0; WTrans := 0; WMang := 0; WBrack := 0;

        J := Round(subsite.OW_Max_Width_Overwash/Site.Scale);
        AdjRow := EachRow; AdjCol := EachCol;
        For I := 1 to J Do   {test m adjacent}
          If AdjCell(AdjRow,AdjCol,True) then Adj(AdjRow, AdjCol, I);

        WaterToLee := WTL and Not RockyFound;
    End; {WaterToLee}
    {------------------------------------------------------------------------------------------------}

Var i: Integer;
Begin {Overwash}
  IF not ERODING then Exit; {run Overwash during erosion portion of calculations}
  If Year < 2025 then exit; {Don't run Overwash when deriving intial conditions}

  {LOOK FOR Overwash CONDITIONS, i.e. Find the front edge of a barrier beach}

  If not CellIsFrontEdge then Exit;  { First we must find the very front edge of the ocean beach }
  If not WaterToLee Then Exit;       { Is there estuary water or marsh Specified distance to lee }

  For i := 1 to subsite.OverWashes_This_dT do
      Overwash;  {Calculate effects}

  {Check for minimum elevation in future versions?}
End; {Overwash}


{----------------------------------------------------}
{ Conversion of Estuary to Beach due to sedimentation}
{ JSC December 2005                                  }
{----------------------------------------------------}

Procedure AggradeBeach(Cell: PCompressedCell);

Var HeightAdded, WidthAdded: Double; {height and width of beach added}
    BTL_Slope, BTL_Distance: Double; {Beach to Lee}
    BTL_Cat: Category; {Beach to Lee}

    {=======================================================================}
    Procedure FindBeachToLee;
    Var BeachFound: Boolean;
        {------------------------------------------------}
        Procedure Adj(Row, Col, I: Integer);
        Var BC: Category;
            C2: CompressedCell;
        Begin
          If BeachFound then exit;
          RetA(Row, Col, C2);
          For BC := EstuarineBeach to OceanFlat do
            Begin
              If CellWidth(@C2,BC) > 0 then
                Begin
                  BeachFound := True;
                  BTL_Distance := ((I-1) * Site.Scale) + CellWidth(@C2,EstuarineWater);
                  If BTL_Distance<0 then BTL_Distance := 0;
                  BTL_Cat := BC;
                  BTL_Slope := C2.TanSlope;
                End;
            End;
        End; {Adj}
        {------------------------------------------------}
    Var I,J: Integer;
        AdjRow,AdjCol: Integer;
    BEGIN
      BeachFound := False;
      J := Round(500.0/Site.Scale);
      AdjRow := EachRow; AdjCol := EachCol;
      Adj(AdjRow, AdjCol, 0); {test cell itself}
      For I := 1 to J Do   {test 500 m adjacent}
        If AdjCell(AdjRow,AdjCol,True) then Adj(AdjRow, AdjCol, I);
    END; {FindBTL}
    {=======================================================================}

Begin                     //  Aggradation currently disabled
  IF ERODING THEN EXIT;

  FindBeachToLee;

  With subsite do
    HeightAdded := 0;
//  HeightAdded := (DeltaT * BeachSed - LocalSLRise) - Depth;  {FIXME}

  WidthAdded := BTL_Distance - (HeightAdded / BTL_Slope);  {FIXME}
  If WidthAdded > 0 then
    Begin
      FracLost[EstuarineWater] :=  WidthAdded / CellWidth(Cell,EstuarineWater);
      {frac. of cat to conv.}         {m}                   {m}
      Convert(Cell,EstuarineWater,BTL_Cat,-99);  {based on sedimentation}
      SetCatElev(Cell,BTL_Cat,MTL);
    End;
End;


{----------------------------------------------------}
{ Convert wetlands due to inundation & erosion       }
{ LowBound is the low elev. allowed for the category }
{ Accrete is Sedimentation for Beaches & Tidal Flats }
{----------------------------------------------------}
Procedure ConvertWetland(Cell: PCompressedCell; Wetland: Category; LowBound: double; Connected_to_Water: Boolean);
Var UpBound, MaxElev: Double;

     {-------------------------------------------------------------}
     PROCEDURE CALCULATE_AGGRADATION;   
     Var ToCat: Category;
     BEGIN
      With Cell^ do
       BEGIN
        MaxElev :=  CatElev(Cell,Wetland) + TanSlope * CellWidth(Cell,Wetland);
        UpBound := UpperBound(Wetland,SubSite);

        If MaxElev > UpBound then
          Begin
            If TanSlope = 0 then FracLost[Wetland] := 1
               else If TanSlope < 0 then FracLost[Wetland] := 0
               else FracLost[Wetland] := ((MaxElev - UpBound) / TanSlope) / CellWidth(Cell,Wetland);
            {frac. of cat to conv.}         {m}      {m}        {unitless}      {m}

            ToCat := UndDryLand;
            If Tropical
              Then Case Wetland of
                     TidalFlat:    ToCat := Mangrove;
                     OceanFlat:    ToCat := OceanBeach;
                     RegFloodMarsh:  ToCat := Mangrove;
                     ScrubShrub, IrregFloodMarsh,
                     EstuarineBeach: ToCat := UndDryland;
                     Mangrove:     ToCat := UndDryland;
                   end {case}
              Else Case Wetland of {Not Tropical}
                     TidalFlat:    ToCat := RegFloodMarsh;
                     OceanFlat:    ToCat := OceanBeach;
                     RegFloodMarsh:  ToCat := ScrubShrub;
                     ScrubShrub, IrregFloodMarsh,
                     EstuarineBeach: ToCat := UndDryland;
                    end; {Case}

            Convert(Cell,Wetland,ToCat,-99);
            SetCatElev(Cell,ToCat,UpBound);
          End; {MaxElev > UpperBound}
       END;  {with}
      END;  {aggradation}
     {-------------------------------------------------------------}

Begin
 With Cell^ do
  Begin
   FracLost[Wetland] := 0.0; {initialization}

   With subsite do
{   IF ((DeltaT * Accrete) > LocalSLRise) and (NOT ERODING) THEN  {aggradation  mod jsc Dec '05}
{     Calculate_Aggradation; }

     If ERODING then
       Begin
         If Wetland = TidalFlat then ErodeTFlat(Cell,Wetland,Connected_to_Water)
         else If ((Erosion > Moderate  {9km fetch}) AND (AdjOcean OR AdjWater)) Then // 4/17/2009
          Case Wetland of
             VegTidalFlat,
              OceanBeach, {not using Bruun}
               EstuarineBeach:  ErodeTFlat(Cell,Wetland,Connected_to_Water);
             RegFloodMarsh,
               IrregFloodMarsh,
               ScrubShrub:    ErodeMarsh(Cell,Wetland);
             Mangrove:        ErodeSwamp(Cell,Wetland);
            end
       End;

     If not ERODING then if Connected_to_Water then
      If (LowBound > CatElev(Cell,Wetland)) then   { Inundation Model }
       BEGIN           {erosion & inundation}
        CalcFracLostBySlope(Cell,Wetland,LowBound);

        { simple inundation}
          Case Wetland of
            VegTidalFlat, TidalFlat:  Convert(Cell,Wetland,EstuarineWater,LowBound);
            EstuarineBeach: Convert(Cell,Wetland,EstuarineWater,LowBound);
            OceanFlat:    Convert(Cell,Wetland,OpenOcean,LowBound);
            RegFloodMarsh:    Convert(Cell,Wetland,TidalFlat,LowBound);
            ScrubShrub, IrregFloodMarsh: Begin
                                           If Tropical
                                             then Convert(Cell,Wetland,Mangrove,LowBound)
                                             else Convert(Cell,Wetland,RegFloodMarsh,LowBound);
                                         End;
            Mangrove:     Convert(Cell,Wetland,EstuarineWater,LowBound);
            end;

          SetCatElev(Cell,Wetland,LowBound);
        END;  {erosion/inundation}

  End; {with Cell}
End; {ConvertWetland}

(********************************************************************************)
(* is exposed water, water, or saltwater adjacent?  i.e. within 500 m off-shore *)
(********************************************************************************)
PROCEDURE TestAdjacent(Cell: PCompressedCell; Row, Col: Integer);

  Procedure Adj1(Iter, NewRow, NewCol: Integer);
  Var MostWater: Boolean;
       C2: CompressedCell;

  BEGIN
    If Not AdjOcean then AdjOcean := GetBit(ExposedWater, NewRow, NewCol);
    MostWater := GetBit(MostlyWater, NewRow, NewCol);
    If Not AdjWater   then AdjWater   := MostWater;
    If Not AdjSalt    then AdjSalt    := GetBit(SaltWater,   NewRow, NewCol);
    ClearPathToOCean := ClearPathToOcean and (MostWater or AdjOcean);
    If (Iter < 3) then  //FIXME, make function of cell-size and maybe erosion rate
      Begin
        If OptimizeLevel < 2
          then
            begin
              RetA(NewRow, NewCol, C2);
              If (C2.MaxFetch > AdjMaxFetch) then AdjMaxFetch := C2.MaxFetch;   {Two immediately adjacent cells are tested}
            end
          else If (MaxFetchArr[(Site.Cols*Row)+EachCol]*0.001  > AdjMaxFetch) then AdjMaxFetch := MaxFetchArr[(Site.Cols*Row)+EachCol]*0.001;
      End;                                {m}            {m/km}     {km}              {km}           {m}                                 {m/km}
  END; {Adj}

Var I, J: Integer;
    AdjRow, AdjCol: Integer;
BEGIN {testadjacent}
  AdjMaxFetch := Cell.MaxFetch;
  If OptimizeLevel > 1 then AdjMaxFetch := MaxFetchArr[(Site.Cols*Row)+EachCol] * 0.001;
  ClearPathToOcean := True;
  AdjOcean := False;
  AdjWater := False;
  AdjSalt  := False;

  J := Round(500.0/Site.Scale);
  AdjRow := Row; AdjCol := Col;
  Adj1(0,AdjRow, AdjCol); {test cell itself}
  ClearPathToOcean := True; {test adjacent cells for path, not cell itself}
  For I := 1 to J Do   {test 500 m adjacent}
    If AdjCell(AdjRow,AdjCol,False) then Adj1(I, AdjRow, AdjCol);

  ClearPathToOcean := ClearPathToOcean and AdjOcean;
END; {TestAdjacent}


PROCEDURE TestNear(Cell: PCompressedCell;VAR NearWater, NearSalt: Boolean);
(************************************************************)
(* is shoreline near?                                       *)
(* What is the nearest wetland, for saturation?}            *)
(* What is the precise distance from the cell to the water? *)
(* Clough, December 2005                                    *)
(************************************************************)

Var I, J: LongInt;
    PreciseDistance: Boolean;
    PrevWetland: Category;
    WetlandWidth: Double;

  {------------------------------------------------}
  Procedure Adj(Row, Col, I: Integer);
  Var Cat: Category;
      C2: CompressedCell;

    {----------------------------------------------}
    Procedure FindNearestWetland;  {Take elevation of nearest 500 m width wetland for water table estimation}
    CONST MIN_WIDTH_WETLAND = 500; {m}
    Begin
     If (WetLandType = Blank) then
       Begin
         RetA(Row, Col, C2);
         Cat := GetCellCat(@C2);

         If Cat in [InlandFreshMarsh,Swamp,CypressSwamp] then  // 6-1-2011, removed Tidal Fresh Marsh, Inland Open Water, and Riverine Tidal from Soil-Saturation effects
           Begin
             If Cat = PrevWetland
               then WetlandWidth := WetlandWidth + Site.Scale
               else WetlandWidth := 0;

             PrevWetland := Cat;

             If WetlandWidth > MIN_WIDTH_WETLAND then
              If (CatElev(@C2,Cat)<10) then  // Avoid perched water tables
                begin
                  WetlandType := Cat;
                  WaterTable := CatElev(@C2,Cat);
                end;
           End;
       End;
     End;
    {----------------------------------------------}

  Begin
    With Cell^ Do
     Begin
       FindNearestWetland;

       If Not NearWater Then NearWater := GetBit(MostlyWater, Row, Col);
       If Not NearSalt Then
         Begin
           NearSalt := GetBit(SaltWater, Row, Col);
           If NearSalt Then
             Begin
               DistancetoWater := I * Site.Scale; {distance from cell to salt water, (Cell-width precision)}
               If PreciseDistance then
                 Begin {precise distance from cell's front edge to open ocean}
                   RetA( Row, Col, C2);
                   If DistancetoWater > 0 then DistancetoWater := DistancetoWater-Site.Scale; {adjust to front edge}
                   DistancetoWater := DistancetoWater + CellWidth(@C2,OceanBeach);
                 End;
             End; {nearsalt}
         End; {not nearsalt}
     End; {with}
  End; {Adj}
  {------------------------------------------------}
Var AdjRow,AdjCol: Integer;
BEGIN
  WaterTable := 0;
  WetLandType := Blank;
  If CellWidth(Cell,DevDryLand) + CellWidth(Cell,UndDryLand) = 0
    then WetLandType := OpenOcean; {avoid unnecessary calculations of water table above}
  PreciseDistance := (CellWidth(Cell,OceanBeach) > 0);  {need precise distance to ocean for erosion calc}
  NearWater := False; NearSalt := False; {initialize}

  J := Round(6000.0/Site.Scale);
  AdjRow := EachRow; AdjCol := EachCol;
  Adj(AdjRow, AdjCol, 0); {test cell itself}
  For I := 1 to J Do   {test 6km adjacent}
    If AdjCell(AdjRow,AdjCol,False) then Adj(AdjRow, AdjCol, I);

END; {TestNear}

PROCEDURE TestLee(Cell: PCompressedCell; VAR NearWater: Boolean);
(************************)
(* is shoreline to lee? *)
(************************)
Var I, J: LongInt;

  {------------------------------------------------}
  Procedure Adj(Row, Col, I: Integer);
  Begin
    If Not NearWater Then
        NearWater := GetBit(MostlyWater, Row, Col);
  End; {Adj}
  {------------------------------------------------}
Var AdjRow,AdjCol: Integer;
BEGIN
  J := Round(6000.0/Site.Scale);
  AdjRow := EachRow; AdjCol := EachCol;
  Adj(AdjRow, AdjCol, 0); {test cell itself}
  For I := 1 to J Do   {test 6km adjacent to lee}
    If AdjCell(AdjRow,AdjCol,True) then Adj(AdjRow, AdjCol, I);

END; {TestLee}

{-------------------------------------------}
{      Transfer percentages of Classes      }
{             MAIN PROCEDURE                }
{        JSC Modified,  1998, 2005-2007     }
{-------------------------------------------}

Var EstNum, i: Integer;
    LocalSLRise: Double;
    Connected_to_Water: boolean;
    CatElev2: Double;
    CellROS: Byte;

Begin  {TRANSFER}
 ERODING := ErodeStep;
 CellFWInfluenced := FWInfluenced(EachRow,EachCol);
 SubSite := Site.GetSubSite(EachCol{x},EachRow{y},Cell);  {Set subsite based on cell being evaluated}

  with Cell^ Do With subsite do
  Begin
   If CellWidth(Cell,Blank) = 0.0 Then    {otherwise skip}
    Begin
     LocalSLRise := SubSite.SLRise - Cell.Uplift * SubSite.DeltaT * 0.01;
      {m}                  {m}           {cm/yr}          {yr}     {cm/m}


     TestAdjacent(Cell, EachRow, EachCol);   {examine adjacent cells}

     Diked := IncludeDikes and ((ClassicDike and Cell.ProtDikes) or
                                ((not ClassicDike) and Cell.ProtDikes and (not Cell.ElevDikes)));

     Erode(Cell,Erosion); {Get qualitative estimate of erosion for this cell based on cell specific maxfetch}

{17} If CellWidth(Cell,EstuarineWater) > 0.0
       Then If Not ERODING Then
         Begin
(*           If (Year > 2000) and ClearPathToOcean then {i.e. if barrier island breached}
             Begin
               FracLost[EstuarineWater] := 1.0;            { (500m    25 years) (20 m / year) }  {Examine, SHOULD PROBABLY REMOVE TIME-STEP SENSITIVITY}
               Convert(Cell, EstuarineWater, OpenOcean,-99);
             End;                                                     Disabled Open Ocean Expansion on 2/24/2011. Not an important prediction.  *)

(*           With subsite do
            If (Year > 2000) AND (Not AdjOcean) AND
//              (Depth < (DeltaT * BeachSed - LocalSLRise))
                Then AggradeBeach(Cell);  FIXME AGGRADATION *)

         End; {EstuarineWater}


     If (CellWidth(Cell,OpenOcean) + CellWidth(Cell,EstuarineWater))/Site.Scale < 1.0 THEN
      BEGIN {if not all water}
        Connected_to_Water := true;
        if checkconnectivity then
          if Connect_Array[(Site.Cols*EachRow)+EachCol]= 4 then
            Connected_to_Water := false;

        TestNear(Cell,NearWater, NearSalt);  {set watertable and wetlandtype}
        If Not NearWater then TestLee(Cell,NearWater);

{14}    If (CellWidth(Cell,RockyIntertidal) > 0.0) and NOT ERODING
          Then If (Connected_to_Water and (CatElev(Cell,RockyIntertidal) <= LowerBound(RockyIntertidal,SubSite))) Then
            If (not Diked) then
              Begin
                 CalcFracLostBySlope(Cell,RockyIntertidal,LowerBound(RockyIntertidal,SubSite));
                 If AdjOcean Then Convert(Cell,RockyIntertidal,OpenOcean,LowerBound(RockyIntertidal,SubSite))
                             Else Convert(Cell,RockyIntertidal,EstuarineWater,LowerBound(RockyIntertidal,SubSite));
               End;

{22}    If (CellWidth(Cell,InlandShore) > 0.0) and NOT ERODING
          Then If (Connected_to_Water and (CatElev(Cell,InlandShore) <= LowerBound(InlandShore,SubSite))) Then
            If (not Diked) then
              Begin
                CalcFracLostBySlope(Cell,InlandShore,LowerBound(InlandShore,Subsite));
                If AdjOcean Then Convert(Cell,InlandShore,OpenOcean,LowerBound(InlandShore,Subsite))
                            Else Convert(Cell,InlandShore,EstuarineWater,LowerBound(InlandShore,Subsite));
              End;

{12}    If CellWidth(Cell,OceanBeach) > 0.0
          Then
             BEGIN
              If Not ERODING then
                  Begin
                    CalcFracLostBySlope(Cell,OceanBeach,LowerBound(OceanBeach,Subsite));
                    Convert(Cell,OceanBeach,OpenOcean,LowerBound(OceanBeach,Subsite));
                  End;
               If ERODING then Begin
                                  If UseBruun then ErodeBeach(Cell)
                                              else ConvertWetland(Cell,OceanBeach,LowerBound(OceanBeach,SubSite),Connected_to_Water);
                               End;
             End;

{10}    If CellWidth(Cell,EstuarineBeach) > 0.0 Then
            ConvertWetland(Cell,EstuarineBeach, LowerBound(EstuarineBeach,SubSite),Connected_to_Water);

{11}    If (CellWidth(Cell,TidalFlat) > 0.0) Then
            ConvertWetland(Cell,TidalFlat, LowerBound(TidalFlat,SubSite),Connected_to_Water); {Tidal Flat}

{13}    If (CellWidth(Cell,OceanFlat) > 0.0) Then                   {Ocean Flat}
            ConvertWetland(Cell, OceanFlat, LowerBound(OceanFlat,SubSite),Connected_to_Water);

{25}    If (CellWidth(Cell,VegTidalFlat) > 0.0) Then
            ConvertWetland(Cell,VegTidalFlat, LowerBound(VegTidalFlat,SubSite),Connected_to_Water); {Veg Tidal Flat}


{9}     If CellWidth(Cell,Mangrove) > 0.0 Then                         {8}
          IF Not Diked {Protected < LowDike} Then
            Begin
              ConvertWetland(Cell,Mangrove, LowerBound(Mangrove,SubSite),Connected_to_Water);
             { If (not NearWater) Then
                Begin
                  FracLost[Mangrove] := 1.0;
                  Convert(Cell,Mangrove,ScrubShrub);
                End;   Talk to Dick, NWI classes converted  TIDAL FLAT PROBLEM  SLAMM 5}
            End;   {mangrove}

{8}     If CellWidth(Cell,RegFloodMarsh) > 0.0 Then
          Begin
            IF Not Diked {Protected < LowDike}
              Then ConvertWetland(Cell,RegFloodMarsh, LowerBound(RegFloodMarsh,SubSite),Connected_to_Water)
              Else DikedWetland(Cell,RegFloodMarsh);
           End;  {RegFloodMarsh}


{7}     If CellWidth(Cell,ScrubShrub) > 0.0 Then                        {7}
          IF Not Diked {Protected < LowDike}
            Then ConvertWetland(Cell,ScrubShrub, LowerBound(ScrubShrub,SubSite),Connected_to_Water)
            Else DikedWetland(Cell,ScrubShrub);

{20}     If CellWidth(Cell,IrregFloodMarsh) > 0.0 Then                        {20}
          IF Not Diked {Protected < LowDike}
            Then ConvertWetland(Cell,IrregFloodMarsh, LowerBound(IrregFloodMarsh,SubSite),Connected_to_Water)
            Else DikedWetland(Cell,IrregFloodMarsh);

{5}     If (CellWidth(Cell,InlandFreshMarsh) > 0.0) and (Not ERODING) Then        {5}
          Begin

            If Diked {Protected < LowDike}
              Then DikedWetland(Cell,InlandFreshMarsh)  {may include rice}
              Else
                Begin
                  If (Connected_to_Water) and (CatElev(Cell,InlandFreshMarsh) < LowerBound(InlandFreshMarsh,SubSite)) Then
                   Begin
                     CalcFracLostBySlope(Cell,InlandFreshMarsh,LowerBound(InlandFreshMarsh,Subsite));
                     If CellFWInfluenced
                       then Convert(Cell,InlandFreshMarsh,TidalFreshMarsh,LowerBound(InlandFreshMarsh,Subsite))  //1-21/2011  alternate flow chart for fw influenced
                       else If (Tropical and NearWater)
                         Then Convert(Cell,InlandFreshMarsh,Mangrove,LowerBound(InlandFreshMarsh,Subsite))
                         Else Convert(Cell,InlandFreshMarsh,ScrubShrub,-99);
                   End; {< LowerBound}
                End; {else not diked}
          End; {InlandFreshMarsh}

{6}     If (CellWidth(Cell,TidalFreshMarsh) > 0.0) and (Not ERODING) Then                       {6}
          Begin
            If Diked {Protected < LowDike}
              Then  DikedWetland(Cell,TidalFreshMarsh)
              Else
                Begin
                   CatElev2 := CatElev(Cell,TidalFreshMarsh);
                   If (CatElev2 < LowerBound(TidalFreshMarsh,SubSite)) Then
                    Begin
                      CalcFracLostbySlope(Cell,TidalFreshMarsh,LowerBound(TidalFreshMarsh,Subsite));
                      If (Tropical and NearWater)
                          Then Convert(Cell,TidalFreshMarsh,Mangrove,LowerBound(TidalFreshMarsh,Subsite))
                          Else Begin
                                 If (CatElev2 >= LowerBound(IrregFloodMarsh,SubSite))               //6.23.2011 Potentially convert tidal fresh marsh directly to salt marsh or tidal flat depending on elevation relative to those classes
                                  then Convert(Cell,TidalFreshMarsh, IrregFloodMarsh,LowerBound(TidalFreshMarsh,Subsite))
                                  else If (CatElev2 >= LowerBound(RegFloodMarsh,SubSite))
                                    then Convert(Cell,TidalFreshMarsh, RegFloodMarsh,LowerBound(TidalFreshMarsh,Subsite))
                                    else Convert(Cell,TidalFreshMarsh, TidalFlat,LowerBound(TidalFreshMarsh,Subsite));
                               End;
                    End; {< LowerBound(,SubSite)}
                End; {not diked}
          End; {TidalFreshMarsh}


{23}     If (CellWidth(Cell,TidalSwamp) > 0.0) Then          {23}
          Begin

            IF Diked {Protected > LowDike}
              Then DikedWetland(Cell,TidalSwamp)
              Else
                Begin
                  FracLost[TidalSwamp] := 0;
                  If ERODING then
                    Begin
                      IF ((Erosion > Moderate) AND (AdjOcean OR AdjWater))  // 4/17/2009
                        Then ErodeSwamp(Cell,TidalSwamp)
                    End;

                  IF NOT ERODING then
                    Begin
                      IF (CatElev(Cell,TidalSwamp) < LowerBound(TidalSwamp,SubSite))
                        Then
                          Begin
                            CalcFracLostbySlope(Cell,TidalSwamp,LowerBound(TidalSwamp,Subsite));
                            If CellFWInfluenced
                              then Convert(Cell,TidalSwamp,TidalFreshMarsh,LowerBound(TidalSwamp,Subsite))  //1-20/2011  alternate flow chart for fw influenced
                              else If (Tropical and NearWater)
                                then Convert(Cell,TidalSwamp,Mangrove,LowerBound(TidalSwamp,Subsite))
                                else Convert(Cell,TidalSwamp,IrregFloodMarsh,LowerBound(TidalSwamp,Subsite));  {6-1-07}
                          End; {< LowerBound}
                    End; {Not ERODING}
                End; {not diked}
          End; {TidalSwamp}

{3}     If CellWidth(Cell,Swamp) > 0.0 Then                            {3}
          Begin
            IF Diked {Protected > LowDike}
              Then DikedWetland(Cell,Swamp)
              Else
                Begin
                  FracLost[Swamp] := 0;
                  If ERODING then
                    Begin
                      IF ((Erosion > Moderate) AND (AdjOcean OR AdjWater))  // 4/17/2009
                        Then ErodeSwamp(Cell,Swamp)
                    End;

                  If NOT ERODING then
                    Begin
                      IF (Connected_to_Water) and (CatElev(Cell,Swamp) < LowerBound(Swamp,SubSite))
                        Then
                          Begin
                            CalcFracLostbySlope(Cell,Swamp,LowerBound(Swamp,Subsite));
                            If CellFWInfluenced
                              then Convert(Cell,Swamp,TidalSwamp,LowerBound(Swamp,Subsite)) //1-20/2011  alternate flow chart for fw influenced
                              else If (Tropical and NearWater)
                                then Convert(Cell,Swamp,Mangrove,LowerBound(Swamp,Subsite))
                                else Convert(Cell,Swamp,ScrubShrub,LowerBound(Swamp,Subsite));
                          End {< LowerBound}
                        Else
                          Begin
                            If CellFWInfluenced
                             then
                               Begin
                                 EstNum := 0;
                                 For i := 0 to NumFWFlows -1 do
                                   If FWInfluenced(EachRow,EachCol,i) then EstNum := i;

                                 If (not FWFlows[EstNum].ExtentOnly) then
                                  If (CatElev(Cell,Swamp) < FWFlows[EstNum].TSElev) then
                                   Begin
                                     CalcFracLostbySlope(Cell,Swamp,FWFlows[EstNum].TSElev);
                                     Convert(Cell,Swamp,TidalSwamp,FWFlows[EstNum].TSElev);
                                   End;
                               End;
                          End;
                    End; {Not ERODING}
                End; {not diked}
          End; {hardwood swamp}

{4}     If CellWidth(Cell,CypressSwamp) > 0.0 Then                            {4}
          Begin
            If Diked {Protected < LowDike}
              Then DikedWetland(Cell,CypressSwamp)
              Else
               Begin
                  FracLost[CypressSwamp] := 0;
                  If ERODING then
                    Begin
                      IF ((Erosion > Moderate) AND (AdjOcean OR AdjWater))  // 4/17/2009
                        Then ErodeSwamp(Cell,CypressSwamp)
                    End;

                  If NOT ERODING then
                    If (Connected_to_Water) and (CatElev(Cell,CypressSwamp) < LowerBound(CypressSwamp,SubSite)) Then
                      Begin
                        CalcFracLostbySlope(Cell,CypressSwamp,LowerBound(CypressSwamp,Subsite));
                        If Tropical and NearWater
                          Then Convert(Cell,CypressSwamp,Mangrove,LowerBound(CypressSwamp,Subsite))
                          Else Convert(Cell,CypressSwamp,EstuarineWater,LowerBound(CypressSwamp,Subsite));
                      End;
               End; {not diked}
          End; {Cypress swamp}

{15}    If (CellWidth(Cell,InlandOpenWater) > 0.0) and (Not ERODING) then
         begin
          If (Connected_to_Water) and (CatElev(Cell,InlandOpenWater) <= LowerBound(InlandOpenWater,SubSite)) Then
            If (not Diked) and AdjSalt then begin
                FracLost[InlandOpenWater] := 1;
                Convert(Cell, InlandOpenWater, EstuarineWater,LowerBound(InlandOpenWater,Subsite));
              end;
          end; {InlandOpenWater}

{16}    If (CellWidth(Cell,RiverineTidal) > 0.0) and (Not ERODING) then
         begin
          If CatElev(Cell,RiverineTidal) <= LowerBound(RiverineTidal,SubSite) Then
            If (not Diked) and AdjSalt then begin
              FracLost[RiverineTidal] := 1;
              Convert(Cell, RiverineTidal, EstuarineWater,LowerBound(RiverineTidal,Subsite));
              end;
         end; {RiverineTidal}

{2}     If (CellWidth(Cell,UndDryland) > 0.0) and (Not ERODING) Then
          Begin

            If (CatElev(Cell,UndDryLand) < LowerBound(UndDryland,SubSite))
            and (not ProtectAll) and (not Diked) and (Connected_to_Water) Then   {Examine, 2M}
              Begin
                  CalcFracLostBySlope(Cell,UndDryLand,LowerBound(UndDryland,Subsite));
                  Inundate(Cell,UndDryland);
              End
             Else If (Year>2000) and (UseSoilSaturation) then {no saturation prior to 2001}
               Begin
                  {Watertable holds minelev of nearest wetland or water from "testnear," must be adjusted}
                  WaterTable := WaterTable + (LocalSLRise / 0.91) * Exp(-0.776 - 0.0012 * DistancetoWater);
                     {m}           {m}                  {m / m}      {          m                  }
                  If (NearSalt And (CatElev(Cell,UndDryland) < WaterTable)) and (WetlandType <> Blank)
                               And (CatElev(Cell,UndDryland) < 10) Then  //10/7/2009 avoid perched water table, no saturation above 10m
                    {Nearsalt procedure sets "wetlandtype" variable}
                    Saturate(Cell, UndDryland, WetlandType);
               End;
            If Diked then DikedWetland(Cell,UndDryLand);
          End;   {UndDryland}

{26}     If (CellWidth(Cell,Backshore) > 0.0) and (Not ERODING) Then
          Begin
            If (CatElev(Cell,Backshore) < LowerBound(Backshore,SubSite))
            and (not Diked) Then   {Examine, 2M}
              Begin
                CalcFracLostBySlope(Cell,Backshore,LowerBound(Backshore,Subsite));
                Inundate(Cell,Backshore);
              End
          End;   {Backshore}


{1}     If (CellWidth(Cell,DevDryland) > 0.0) and (Not ERODING) and (Not ProtectAll) and (Not ProtectDeveloped) Then
          Begin
          
            If (CatElev(Cell,DevDryland) < LowerBound(DevDryland,SubSite))
            and  (not Diked) and (Connected_to_Water) Then  {Examine, 2M}
              Begin
                CalcFracLostBySlope(Cell,DevDryLand,LowerBound(DevDryland,Subsite));
                Inundate(Cell,DevDryland);
               End;

            If Diked then DikedWetland(Cell,UndDryLand);
(*             Begin
                  {Assume no saturation of developed dry land}
                  {Watertable holds minelev of nearest wetland or water, it is now adjusted}
                  WaterTable := WaterTable + (LocalSLRise / 0.91) * Exp(-0.776 - 0.0012 * DistancetoWater);
                     {m}           {m}                 {m / m}      {          m                  }
                  If (NearSalt And (CatElev(Cell,DevDryland) < WaterTable)) and (WetlandType <> Blank) Then
                    Saturate(Cell, DevDryland, WetlandType);   ASSUME NO SOIL SATURATION OF DEVELOPED LAND DUE TO CONSTRUCTION OF DRAINAGE CANALS / BRINGING OF FILL 2/24/06
               End; *)
          End;   {DevDryland}

{End Classes}

       If ERODING then TestOverwash(Cell);

       if ERODING then
         // Convert cell by salinity if
         // (a) is influenced by fresh water flows and salinity is defined OR
         // (b) there is a salinity raster and salinity is defined in the cell
         if (CellFWInfluenced and (Cell.Sal[NUM_SAL_METRICS]>-1)) or ((trim(SalFileN) <> '') and (Cell.Sal[1]>-1)) then ConvertCellbySalinity(Cell,SalRules);
                                   //check salinity at 30D level                                 //check sal[1] used by SalFile algorithm
       If USE_DATAELEV and (OptimizeLevel>1)
          then If DataElev[(Site.Cols*EachRow)+EachCol] > Round(SLRise*0.001)
                  then DataElev[(Site.Cols*EachRow)+EachCol] := DataElev[(Site.Cols*EachRow)+EachCol] - Round(SLRise*0.001)
                                     {mm}                                   {mm}                             {mm/delta t}
                  else DataElev[(Site.Cols*EachRow)+EachCol] := 0;

      END; {not all water}
    END;  {not blank}

    If ERODING then
      Begin
         For i := 0 to Site.NOutputSites do
           For CC := FirstCategory to LastCategory Do
             If (i=0) or Site.InOutSite(EachCol,EachRow,i) then
               CatSums[i,CC] := CatSums[i,CC] + (CellWidth(Cell,CC) * Site.Scale);
                 {m2}                {m2}             {m}              {m}

         if ROSArray <> nil then
           Begin
             CellROS :=  ROSArray[(Site.Cols*EachRow)+EachCol];
             For i := 1 to Site.MaxROS do
               For CC := FirstCategory to LastCategory Do
                 If (i=CellROS) then
                   CatSums[i+Site.NOutputSites,CC] := CatSums[i+Site.NOutputSites,CC] + (CellWidth(Cell,CC) * Site.Scale);
                     {m2}                                                       {m2}             {m}              {m}
           End;  {ROSArray<>nil}
      End; {Eroding}
  End; {WITH Cell}

End; {TRANSFER}



{*******************************************}
{        Change attributes for water        }
{         R. A. Park    3/18/88             }
{  Changed to process entire map prior to   }
{  call of Transfer for each cell. Off-Shore}
{  direction should have no effect          }
{                                           }
{  Proc. Adds Accretion to Data Elev.       }
{                                           }
{             J.Clough 12/2005              }
{*******************************************}

Function TSLAMM_Simulation.ChngWater(CalcMF, CalcGeometry: Boolean): Boolean;
{This procedure sets the current cell's water attribute settings
   ExposedWater (50% open ocean)
   MostlyWater  (90% water)
   SaltWater    (>10% salt characteristics)
   AlLWetLand   (<10% dry land) }
Var ER, EC: Integer;
    Cat: Category;
    Acc: Double;
    CWC: CompressedCell;
    SubSite: TSubSite;

Begin
  Result := True;
  If (Length(BMatrix) < Site.Rows * Site.Cols) then
    Begin
      BMatrix := nil;
      SetLength (BMatrix,Site.Rows * Site.Cols);
    End;

  If (Length(ErodeMatrix) < Site.Rows * Site.Cols) then
    Begin
      ErodeMatrix := nil;
      SetLength (ErodeMatrix,Site.Rows * Site.Cols);
    End;

 FOR ER := 0 To (Site.Rows-1) Do
  FOR EC := 0 To (Site.Cols-1) Do
   Begin
     If EC=1 then
       begin
         Result := ProgForm.Update2Gages(Trunc((Er)/Site.Rows*100));
         ProgForm.ProgressLabel.Caption:='Updating Water Attributes (Chngwater)';
         If Not Result then Exit;
       end;

    BMatrix[Site.Cols*(ER)+(EC)]:=0;
    ErodeMatrix[Site.Cols*(ER)+(EC)]:=0;


    RetA(ER, EC, CWC);
    Subsite := Site.GetSubSite(EC {x},ER {y},@CWC);

    With CWC Do With subsite do
      Begin
        If (CellWidth(@CWC,OpenOcean)/Site.Scale > 0.5)
            Then SetBit(ExposedWater,ER,EC,True)
            Else SetBit(ExposedWater,ER,EC,False);

        IF ((CellWidth(@CWC,OpenOcean) + CellWidth(@CWC,EstuarineWater)
             + CellWidth(@CWC,RiverineTidal))/Site.Scale) > 0.9
            Then SetBit(MostlyWater,ER,EC,TRUE)
            Else SetBit(MostlyWater,ER,EC,FALSE);

        If (CellWidth(@CWC,Mangrove) + CellWidth(@CWC,RegFloodMarsh) + CellWidth(@CWC,ScrubShrub) +
           + CellWidth(@CWC,EstuarineBeach)+CellWidth(@CWC,TidalFlat)+CellWidth(@CWC,OceanBeach)+CellWidth(@CWC,IrregFloodMarsh)
           + CellWidth(@CWC,OceanFlat)+CellWidth(@CWC,OpenOcean))/Site.Scale > 0.1
           Then SetBit(Saltwater,ER,EC,TRUE) {saltwetlands occur or salt water}
           Else SetBit(Saltwater,ER,EC,FALSE);

        IF ((CellWidth(@CWC,DevDryland) + CellWidth(@CWC,UndDryland))/Site.Scale < 0.1)
           Then SetBit(AllWetland,ER,EC,True)
           Else SetBit(AllWetland,ER,EC,False);

        { ADD ACCRETION TO "DATA ELEVATION" USED IN SALINITY CALCULATIONS }
        Acc := 0;
        Cat :=  GetCellCat(@CWC);
        If Cat = Mangrove         then Acc := MangroveAccr * 0.001;  {MANGROVE ACCRETION}
        If Cat = RegFloodMarsh    then ACC := RegFloodAccrete(@CWC,SubSite)   *0.001;
        If Cat = TidalFreshMarsh  then ACC := TidalFreshAccrete(@CWC,SubSite) *0.001;
        If Cat in [IrregFloodMarsh, ScrubShrub] then ACC := IrregFloodAccrete(@CWC,SubSite) * 0.001;
        If Cat = InlandFreshMarsh then ACC := InlandFreshAccr * 0.001;
        If Cat in [Swamp,CypressSwamp] then ACC := SwampAccr  * 0.001;
        If Cat = TidalSwamp            then ACC := TSwampAccr * 0.001;

        If USE_DATAELEV and (OptimizeLevel>1) and (Acc > 0)
           then DataElev[(Site.Cols*ER)+EC] := DataElev[(Site.Cols*ER)+EC] + Round((DeltaT*Acc) * 0.001)
                           {mm}                                 {mm}                 {yr}  {m/yr}
      End; {with}

    // If ACC>0 then SetA(ER, EC, CWC);

   End;

  If CalcMF then Result := CalculateMaxFetch;
  If Not Result then Exit;

  If CalcGeometry then Result := CalculateFlowGeometry(Year);
End; {ChngWater}


{*******************************************}
{ Added Calculation of MaxFetch as cell     }
{ attribute     J.Clough 12/2005            }
{*******************************************}

Function TSLAMM_Simulation.CalculateMaxFetch: Boolean;
CONST HIGH_FETCH = 20.1;  {maximum relevant fetch ... infinite}
Var ER, EC: Integer;
    CompassPoints: Integer;
    MFC: CompressedCell;

    {---------------------------------------------------------------------}
    Function ValidCell(R,C: Integer): Boolean;
    Begin
      ValidCell := (R>=0) and (C>=0) and (R<Site.Rows) and (C<Site.Cols);
    End;
    {---------------------------------------------------------------------}
    Procedure CheckFetch(DX,DY:Integer; Incr: Double);
    Var ThisFetch: Double;
        ClearPath: Boolean;
        CurrX, CurrY: Integer;
        OverOcean: Boolean;
        {---------------------------------------------------------------------}
        Function CheckCell(R,C:Integer): Boolean;
        Var C2: CompressedCell;
        Begin
          If Not ValidCell(R,C)
            Then Begin
                   CheckCell := False;
                   If GetBit(ExposedWater,CurrX,CurrY) then MFC.MaxFetch := HIGH_FETCH;
                   {out of bounds in the ocean, assume maximum fetch is infinite}
                 End
            Else Begin
                   If GetBit(ExposedWater,R,C) then OverOcean := True;
                   CheckCell := GetBit(MostlyWater,R,C);
                   If (Not Result) and OverOcean then
                    {If blank after passing through ocean, assume max fetch is infinite}
                     Begin
                       RetA( R, C, C2);
                       If CellWidth(@C2,Blank) > 0 then MFC.MaxFetch := HIGH_FETCH;
                     End;
                 End;
        End;
        {---------------------------------------------------------------------}
    Begin
      If MFC.MaxFetch>20 then exit;
      ThisFetch := 0;
      CurrX := ER;
      CurrY := EC;
      ClearPath := True;
      OverOcean := False;

      Repeat
        If Abs(DX)>1 then {check intermediate cells too}
           ClearPath := CheckCell(CurrX+(DX DIV 2),CurrY) and CheckCell(CurrX+DX,CurrY);
        If Abs(DY)>1 then {check intermediate cells too}
           ClearPath := CheckCell(CurrX,CurrY+(DY DIV 2)) and CheckCell(CurrX,CurrY+DY);

        If ClearPath then ClearPath:=CheckCell(CurrX+DX,CurrY+DY);
        If ClearPath then
          Begin
            ThisFetch := ThisFetch + Incr  * Site.Scale * 0.001;
             {km}          {km}      {cells}     {m/cell}   {m/km}
            CurrX := CurrX + DX;
            CurrY := CurrY + DY;
          End;

      Until Not ClearPath;

      If ThisFetch > MFC.MaxFetch then MFC.MaxFetch := ThisFetch;
    End;
    {---------------------------------------------------------------------}
    Function MiddleOfWater(R,C: Integer): Boolean;
    Var W1,W2,W3,W4 : Boolean;
    Begin
      MiddleOfWater := False;
      If not GetBit(MostlyWater,R,C) then Exit;  {cell itself must be water}
      If ValidCell(R+1,C) and ValidCell(R-1,C) and
        ValidCell(R,C+1) and ValidCell(R,C-1) then
          Begin                                 {and four surrounding cells}
            W1 := GetBit(MostlyWater,R-1,C);
            W2 := GetBit(MostlyWater,R+1,C);
            W3 := GetBit(MostlyWater,R,C-1);
            W4 := GetBit(MostlyWater,R,C+1);
            MiddleOfWater := W1 and W2 and W3 and W4;
          End;
    End;
    {---------------------------------------------------------------------}

Var SaveCaption: String;
Begin
 SaveCaption := ProgForm.ProgressLabel.Caption;
 ProgForm.ProgressLabel.Caption:='Calculating Maximum Fetch';

 FOR ER := 0 To (Site.Rows-1) Do
  FOR EC := 0 To (Site.Cols-1) Do
   Begin
    If EC=1 then
     Begin
      Result := ProgForm.Update2Gages(Trunc((Er-1)/Site.Rows*100));
      If Not Result then Exit;
     End;

    If not MiddleOfWater(ER,EC) then
      Begin
        RetA(ER, EC, MFC);
        MFC.MaxFetch := 0;
        FOR CompassPoints := 1 to 16 do
            Case CompassPoints of
              1:  CheckFetch(1,0,1);
              5:  CheckFetch(2,1,2.236);
              9:  CheckFetch(1,1,1.412);
              13: CheckFetch(1,2,2.236);
              2:  CheckFetch(0,1,1);
              6:  CheckFetch(-1,2,2.236);
              7:  CheckFetch(-1,1,1.412);
              8:  CheckFetch(-2,1,2.236);
              3:  CheckFetch(-1,0,1);
              10: CheckFetch(-2,-1,2.236);
              11: CheckFetch(-1,-1,1.412);
              12: CheckFetch(-1,-2,2.236);
              4:  CheckFetch(0,-1,1);
              14: CheckFetch(1,-2,2.236);
              15: CheckFetch(1,-1,1.412);
              16: CheckFetch(2,-1,2.236);
              End; {Case}

        If OptimizeLevel>1 then
          If MFC.MaxFetch > 65.5 then MaxFetchArr[(Site.Cols*ER)+EC] := 65500
                                 else MaxFetchArr[(Site.Cols*ER)+EC] := Round(MFC.MaxFetch*1000);
        SetA(ER, EC, MFC);
      End;
   End; {for EC}

  ProgForm.ProgressLabel.Caption:=SaveCaption;
  Result := ProgForm.Update2Gages(0);  
End; {CalculateMaxFetch}
