//SLAMM SOURCE CODE copyright (c) 2009 - 2012 Warren Pinnacle Consulting, Inc.
//Code Use and Redistribution is Subject to Licensing, SEE SLAMM_License.txt

unit Uncert;

interface

Uses Classes, CalcDist, UncertDefn;

Type

SingleArray = Array of Single;

TSLAMM_Uncertainty = Class
   NumDists: Integer;
   DistArray: TDistArray;

   NumSens: Integer;
   SensArray: TSensArray;
   PctToVary: Integer;

   Seed, Iterations, GIS_Start_Num: Integer;
   UseSeed: Boolean;

  {No load or save below}

   ZUncertMap: Array[1..2] of SingleArray;
   NMapIter: Integer;
   XLS_Path, CSV_Path : String;
   UncSens_Iter, UncSens_Row: Integer;
   SensPlusLoop: Boolean;

   Procedure MakeUncertMap(MapNum,NR,NC: Integer; RMSE,q_val: Double);
   Constructor Create;
   Destructor Destroy;  override;
   Constructor Load(ReadVersionNum: Double; TS: TStream; PSS: Pointer);
   Procedure Store(Var TS: TStream);
End;

implementation

Uses SysUtils, Global, Progress, Forms;

{ TSLAMM_Uncertainty }

constructor TSLAMM_Uncertainty.Create;
begin
  NumDists := 0;
  DistArray := nil;

  NumSens := 0;
  SensArray := nil;
  PctToVary := 15;

  Seed       := 20;
  Iterations := 20;
  GIS_Start_Num := 1;
  UseSeed    := True;

  UncSens_Iter := 0;
  UncSens_Row := 0;
  NMapIter := 0;
end;

destructor TSLAMM_Uncertainty.Destroy;
Begin
  Finalize(DistArray);
  Finalize(SensArray);
  Finalize(ZUncertMap);
End;


procedure TSLAMM_Uncertainty.Store(var TS: TStream);
var i: Integer;
begin
  TSWrite('NumDists',NumDists);
  For i:=0 to NumDists-1 do
    DistArray[i].Store(TS);

  TSWrite('Seed',Seed);
  TSWrite('Iterations',Iterations);
  TSWrite('GIS_Start_Num',GIS_Start_Num);

  TSWrite('UseSeed',UseSeed);

  TSWrite('NumSens',NumSens);
  TSWrite('PctToVary',PctToVary);
  For i:=0 to NumSens-1 do
    SensArray[i].Store(TS);


end;


constructor TSLAMM_Uncertainty.Load(ReadVersionNum: Double; TS: TStream; PSS: Pointer);
Var i: Integer;
    JunkBool: Boolean; JunkStr: String;
begin
  TSRead('NumDists',NumDists);
  SetLength(DistArray,NumDists);
  For i:=0 to NumDists-1 do
    DistArray[i] := TInputDist.Load(ReadVersionNum,TS,PSS);

  TSRead('Seed',Seed);
  TSRead('Iterations',Iterations);
  If ReadVersionNum > 6.315 then TSRead('GIS_Start_Num',GIS_Start_Num)
                            else GIS_Start_Num := 1;

  TSRead('UseSeed',UseSeed);

  If ReadVersionNum < 6.325 then
    Begin
      TSRead('SaveUncGIS',JunkBool);
      TSRead('SaveALLGIS',JunkBool);
      TSRead('GISYears',JunkStr,ReadVersionNum);
    End;  

  If ReadVersionNum > 6.215
    then
      Begin
        TSRead('NumSens',NumSens);
        TSRead('PctToVary',PctToVary);
        SetLength(SensArray,NumSens);
        For i:=0 to NumSens-1 do
          SensArray[i] := TSensParam.Load(ReadVersionNum,TS,PSS);
       End
    else
      Begin
        NumSens := 0;
        SensArray := nil;
        PctToVary := 15;
      End;

  {No load or save below}
  UncSens_Iter := 0;
  UncSens_Row := 0;
  For i := 1 to 2 do
    ZUncertMap[i] := nil;
  NMapIter := 0;

end;

procedure TSLAMM_Uncertainty.MakeUncertMap(MapNum, NR, NC: Integer;  RMSE, q_Val: Double);

Const R_Val = 0.2;

Var ER, EC: Integer;
    MaxChange: Single;
    ZMap, PrevZMap: Array of Single;
    Index: Integer;
    Tolerance: Double;
    WasVisible: Boolean;
//    TextOut: TextFile;   // Debug text file to test correlations
//    Appending: Boolean;

    Function GetZ(ER,EC:Integer): Single;
    Begin
      If (ER<0) or (EC<0) or (ER>NR-1) or (EC>NC-1) then GetZ :=0
         else GetZ := PrevZMap[NC*(ER)+(EC)]
    End;

Var  StDevCorrection, Sum_e2: Double;

begin
  WasVisible := ProgForm.Visible;
  ProgForm.Visible := True;
  ProgForm.YearLabel.Visible := False;
  ProgForm.SLRLabel.Visible := False;
  ProgForm.ProtectionLabel.Visible := False;

  ProgForm.ProgressLabel.Caption   := 'Calculating Elevation Uncertainty Map';


  Tolerance := RMSE / 100;   {Tolerance in variability error set to 1% of RMSE}

  If (q_Val >= 0.25) or (q_Val < 0) then
    Raise ESLAMMError.Create('Spatial Correlation Q must be less than 0.25 and greater than or equal to 0');

// Appending := FileExists('C:\newtemp\ArrayOut.csv');
//  AssignFile(TextOut,'C:\newtemp\ArrayOut.csv');
//  If Appending
//        then Append(TextOut)
//        else Rewrite(TextOut);

//  Writeln(TextOut);
//  Writeln(TextOut,'Q-Val ',q_Val);

  If (Length(ZUncertMap) < NR*NC) then SetLength(ZUncertMap[MapNum], NR*NC);
  SetLength(ZMap, NR*NC);
  SetLength(PrevZMap, NR*NC);

  // Initialize with Normal Distribution

{ the RMSE is an estimator of the standard deviation based on model results.
  If it is an unbiased estimator, then it will be equal to the standard error. }
  For ER := 0 to NR-1 do
     For EC := 0 to NC-1 do
      Begin
        Index :=NC*(ER)+(EC) ;
        ZUncertMap[MapNum][Index] := rNormal(0,  RMSE);
        PrevZMap[Index] := R_Val * ZUncertMap[MapNum][Index];
      End;

  NMapIter := 0;
 // Iterate until appropriate spatial variance is achieved
  Repeat
    inc(NMapIter);
    For ER := 0 to NR-1 do
     For EC := 0 to NC-1 do
      ZMap[NC*(ER)+(EC)] := q_Val*( GetZ(ER-1,EC)+GetZ(ER+1,EC)+GetZ(ER,EC-1)+GetZ(ER,EC+1)) + ZUncertMap[MapNum][NC*(ER)+(EC)]*R_Val;

    MaxChange := -9999;
     For ER := 0 to NR-1 do
      For EC := 0 to NC-1 do
       Begin
        Index := NC*(ER)+(EC);
        If ABS(ZMap[Index]-PrevZMap[Index]) > MaxChange then MaxChange := ABS(ZMap[Index]-PrevZMap[Index]);
        PrevZMap[Index] := ZMap[Index];
       End;

    ProgForm.Gauge1.Progress := NMapIter - 100*(NMapIter Div 100);
    Application.ProcessMessages;

  Until MaxChange < Tolerance;

  Sum_e2 := 0;
  For ER := 0 to NR-1 do
   For EC := 0 to NC-1 do
    Begin
      Index := NC*(ER)+(EC);
      ZUncertMap [MapNum][Index] := ZUncertmap[MapNum][Index] + ZMap [Index];
      Sum_e2 := Sum_e2 + Sqr(ZUncertmap [MapNum][Index]);
    End;

 // Correct standard devation
  StDevCorrection := RMSE / SQRT(Sum_e2 / (NR*NC-1));
  For ER := 0 to NR-1 do
   Begin
//    Writeln(TextOut);
    For EC := 0 to NC-1 do
     Begin
       Index := NC*(ER)+(EC);
       ZUncertMap [MapNum][Index] := ZUncertmap [MapNum][Index] * StDevCorrection;
//       Writeln(TextOut,IntToStr(ER),' ',IntToStr(EC),' ',FloatToStrF(ZUncertmap [MapNum][Index],ffgeneral,5,3));
     End;
   End;

  ZMap := nil;
  PrevZMap := nil;
  ProgForm.Visible := WasVisible;
  ProgForm.YearLabel.Visible := True;
  ProgForm.SLRLabel.Visible := True;
  ProgForm.ProtectionLabel.Visible := True;

//  Closefile(TextOut)
end;


end.
