unit TCollect;
interface
uses classes,sysutils,graphics;

{ This entire library borrowed from BP 7.0 and then modified to exclude
  assembly language to provide 16-32 bit compatibility by JSC           }

{ Duplicates are turned off in the sorted list and ForEach was not coded }

Type
{ TObject base object }
  TAquatoxError = Exception;

  POldObject = ^TOldObject;
  TOldObject = object
    constructor Init;
    procedure Free;
    destructor Done; virtual;
    function ObjectID : SmallInt; virtual;
    procedure Store(Var St: TStream); virtual;
  end;


const MaxCollectionSize=10000;
type

{ TCollection types }

  PItemList = ^TItemList;
  TItemList = array[0..MaxCollectionSize - 1] of Pointer;

{ TCollection object }

  PCollection = ^TCollection;
  TCollection = object(TOldObject)
    Items: PItemList;
    Count: Integer;
    Limit: Integer;
    Delta: Integer;
    LastIndexRead: Integer;  {last index read, for optimization...  does not req. save}
    constructor Init(ALimit, ADelta: Integer);
    constructor Load(Var S: Tstream);
    destructor Done; virtual;
    destructor SavePointers; virtual;
    function At(Index: Integer): Pointer;
    procedure AtFree(Index: Integer);
    procedure AtInsert(Index: Integer; Item: Pointer);
    procedure AtPut(Index: Integer; Item: Pointer);
    procedure FreeAll;
    procedure FreeItem(Item: Pointer); virtual;
    function  IndexOf(Item: Pointer): Integer; virtual;
    procedure Insert(Item: Pointer); virtual;
    procedure SetLimit(ALimit: Integer); virtual;
    procedure Store(var S: TStream); virtual;
  end;

{ TSortedCollection object }

  PSortedCollection = ^TSortedCollection;
  TSortedCollection = object(TCollection)
    Duplicates: Boolean;
    constructor Init(ALimit, ADelta: Integer);
    constructor Load(Var S: Tstream);
    function Compare(Key1, Key2: Pointer): Integer; virtual;
    function IndexOf(Item: Pointer): Integer; virtual;
    procedure Insert(Item: Pointer); virtual;
    function KeyOf(Item: Pointer): Pointer; virtual;
    function Search(Key: Pointer; var Index: Integer): Boolean; virtual;
    procedure Store(var S: TStream); virtual;
  end;


implementation

uses WinProcs;

{ TObject }

constructor TOldObject.Init;
begin
end;


{ Shorthand procedure for a done/dispose }

procedure TOldObject.Free;
begin
  If @Self<>nil then
    Dispose(POldObject(@Self), Done);
end;

destructor TOldObject.Done;
begin

end;

function TOldObject.ObjectID: SmallInt;
begin
  ObjectID:=-1;
end;

Procedure TOldObject.Store(Var St: TStream);
begin
   raise taquatoxerror.create('Base Object Store Invoked');
end;

{ TCollection }

constructor TCollection.Init(ALimit, ADelta: Integer);
begin
  TOldObject.Init;
  Items := nil;
  Count := 0;
  Limit := 0;
  Delta := ADelta;
  SetLimit(ALimit);
  LastIndexRead := -1;
end;

constructor TCollection.Load(Var S: Tstream);

var
  Short: SmallInt;
  I,C : Integer;
begin
  LastIndexRead := -1;
  Short:=0;
  S.Read(Short, 6);
  Items := nil;
  Count:=Short;
  I := Count;
  C := Count;
  Delta:=5;
  Limit := 0;
  Count := 0;
  SetLimit(I);
  Count := C;
end;

destructor TCollection.Done;
begin
  FreeAll;
  SetLimit(0);
end;

destructor TCollection.SavePointers;
Begin
  SetLimit(0);
End;

function TCollection.At(Index: Integer): Pointer;

begin
  If (Index>(Count-1)) or (Index<0) then
    raise taquatoxerror.create('Collection Boundary Exceeded on AT');
  At:=Items[Index];
end;

procedure TCollection.AtFree(Index: Integer);
Var Loop: Integer;
begin
  If (Index>(Count-1)) or (Index<0) then raise taquatoxerror.create('Collection Boundary Exceeded on Free');

  FreeItem(Items[index]);

  For Loop:=Index+1 to Count-1 do
      Items[Loop-1]:=Items[Loop];

  Count:=Count-1;
end;

procedure TCollection.AtInsert(Index: Integer; Item: Pointer);
Var Loop : Integer;

begin
  If (Index>(Count)) or (Index<0) then raise taquatoxerror.create('Collection Boundary Exceeded on Insert');
  If Count=Limit then SetLimit(Limit+Delta);
  Count:=Count+1;

  For Loop:=Count-1 downto Index+1 do
      Items[Loop]:=Items[loop-1];

  AtPut(Index,Item);
end;

procedure TCollection.AtPut(Index: Integer; Item: Pointer);

begin
  If (Index>(Count-1)) or (Index<0) then raise taquatoxerror.create('Collection Boundary Exceeded on Put');
  Items[Index]:=Item;
end;

procedure TCollection.FreeAll;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do FreeItem(At(I));
  Count := 0;
end;

procedure TCollection.FreeItem(Item: Pointer);
begin
  if Item <> nil then Dispose(POldObject(Item), Done);
end;

function TCollection.IndexOf(Item: Pointer): Integer;
Var Loop: Integer;
begin
  Indexof := -1;
  For Loop:=0 to count-1 do
      If At(Loop)=Item then begin Indexof:=Loop; exit; end;

end;

procedure TCollection.Insert(Item: Pointer);
begin
  AtInsert(Count, Item);
end;

procedure TCollection.SetLimit(ALimit: Integer);
var
  AItems: PItemList;
begin
  if ALimit < Count then ALimit := Count;
  if ALimit > MaxCollectionSize then ALimit := MaxCollectionSize;
  if ALimit <> Limit then
  begin
    if ALimit = 0 then AItems := nil else
    begin
      GetMem(AItems, ALimit * SizeOf(Pointer));
      if (Count <> 0) and (Items <> nil) then
        Move(Items^, AItems^, Count * SizeOf(Pointer));
    end;
    if Limit <> 0 then FreeMem(Items, Limit * SizeOf(Pointer));
    Items := AItems;
    Limit := ALimit;
  end;
end;


procedure TCollection.Store(var S: TStream);
Var SmCt: SmallInt;
begin
  SmCt:=Count;
  S.Write(SmCt, 6);
end;

{ TSortedCollection }

constructor TSortedCollection.Init(ALimit, ADelta: Integer);
begin
  TCollection.Init(ALimit, ADelta);
  Duplicates := False;
end;

constructor TSortedCollection.Load(Var S: Tstream);
begin
  TCollection.Load(S);
  Duplicates := False;
end;

procedure TSortedCollection.Store(var S: TStream);
begin
  TCollection.Store(S);
end;


function TSortedCollection.Compare(Key1, Key2: Pointer): Integer;
begin
   Compare:=0;
end;

function TSortedCollection.IndexOf(Item: Pointer): Integer;
{gives index of a given item in the list}
var
  I: Integer;
begin
  IndexOf := -1;
  if Search(KeyOf(Item), I) then
  begin
    if Duplicates then
      while (I < Count) and (Item <> Items^[I]) do Inc(I);
    if I < Count then IndexOf := I;
  end;
end;

procedure TSortedCollection.Insert(Item: Pointer);
var
  I: Integer;
begin
  If Search(KeyOf(Item),I) and (not Duplicates) then raise taquatoxerror.create('Programming Error: Duplicate Items in This Sorted Collection are not allowed')
                                                else AtInsert(I, Item);
end;

function TSortedCollection.KeyOf(Item: Pointer): Pointer;
begin
  KeyOf := Item;
end;

function TSortedCollection.Search(Key: Pointer; var Index: Integer): Boolean;
{if duplicates, gives the index of the first item in the list}

var
  L, H, I, C: Integer;
begin
  Search := False;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) div 2;
    C := Compare(KeyOf(Items^[I]), Key);
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
      begin
        Search := True;
        if not Duplicates then L := I;
        if Duplicates then while Compare(Keyof(Items^[L]),Key)<>0 do
            inc(L);
      end;
    end;
  end;
  Index := L;
end;


end.

