unit UncertMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, SLR6,
  StdCtrls, ExtCtrls, UncertDefn, DistEdit, Menus, CalcDist, RandNum, TCollect,
  Buttons;

type
  TUncertForm = class(TForm)
    ExecuteButton: TButton;
    AddDist: TButton;
    DistBox: TListBox;
    RemoveDist: TButton;
    EditDist: TButton;
    Panel1: TPanel;
    Label2: TLabel;
    IterEdit: TEdit;
    SeedLabel: TLabel;
    SeedEdit: TEdit;
    Label1: TLabel;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    MainMenu1: TMainMenu;
    UseSeedBox: TRadioButton;
    UseRandomSeed: TRadioButton;
    FileMenu: TMenuItem;
    Load1: TMenuItem;
    Save1: TMenuItem;
    Close1: TMenuItem;
    Bevel1: TBevel;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    UpBtn: TBitBtn;
    DownBtn: TBitBtn;
    Label4: TLabel;
    SensPanel: TPanel;
    Label5: TLabel;
    PctVaryEdit: TEdit;
    Label6: TLabel;
    GISNumEdit: TEdit;
    HelpButton: TBitBtn;
    procedure ExecuteButtonClick(Sender: TObject);
    procedure AddDistClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EditDistClick(Sender: TObject);
    procedure RemoveDistClick(Sender: TObject);
    procedure DistBoxKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure UseSeedBoxClick(Sender: TObject);
    procedure ConvInt(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure Load1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure HelpButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    Sim: TSLAMM_Simulation;
    Changed: Boolean;
    SensMode: Boolean;
    ShellPath: String;
    Procedure NewShell;
    Procedure UpdateScreen;
    Procedure SaveShell;
    Procedure LoadShell;
    Procedure EditUncertSetup(TSS: TSLAMM_Simulation);
  end;

{    Const NumOutputs = 25;
          SiteToWrite = 0; }

var
  UncertForm: TUncertForm;

implementation

uses selectinput, Progress, Uncert, Main, Global, System.UITypes, Execute;

{$R *.DFM}

procedure TUncertForm.EditUncertSetup;
Begin
  Sim := TSS;
  UpdateScreen;
  ShowModal;

End;

procedure TUncertForm.ExecuteButtonClick(Sender: TObject);
begin
(*  Uncert_GISYear := StrToInt(OutputDateEdit.Text);
  If Iterations<2 then Begin Messagedlg('You must run at least two iterations',mterror,[mbok],0); exit; end;
  If NumDists=0 then Begin Messagedlg('You must distribute at least one input parameter',mterror,[mbok],0); exit; end;

  SaveDialog1.Title:='Select Excel File to Save Output to:';
  SaveDialog1.Filter := 'Excel files (*.xls)|*.xls';
  SaveDialog1.FileName := '';
  If not SaveDialog1.Execute then exit;
  XLSPath := ChangeFileExt(SaveDialog1.FileName, '.xls');
  If FileExists(XLSPath) and Not DeleteFile(XLSPath) then Begin Messagedlg('The XLS File you chose appears to be in use by another application.',mterror,[mbok],0); exit; end;
                                                    *)
End;





procedure TUncertForm.AddDistClick(Sender: TObject);

    Procedure AddParam(PID: Integer);
    Var j: Integer;
        NewSens: TSensParam;

    Begin
     If (PID = 2) or (PID = 5)
                then Begin
                       MessageDlg('Elevation Uncertainty is a spatial parameter, and is not relevant for sensitivity analysis.',mterror,[mbok],0);
                       Exit;
                     End;

     With Sim.UncertSetup do
      Begin
        For j := 0 to NumSens -1 do
           If SensArray[j].IDNum = PID then Exit;

        NewSens := TSensParam.Create(PID,Sim );
        Inc(NumSens);
        SetLength(SensArray,NumSens);
        SensArray[NumSens-1] := NewSens;
        UpdateScreen;
       End; {with}
    End; {AddParam}   

    Procedure AddAndEdit(PID: Integer);
    Var j: Integer;
        NewDist: TInputDist;

    Begin
     With Sim.UncertSetup do
      Begin
        For j := 0 to NumDists-1 do
           If DistArray[j].IDNum = PID then Exit;

        NewDist := TInputDist.Create(PID,Sim );

        DistributionForm.PDistArr := @DistArray;
        DistributionForm.PNumDists := @NumDists;

        If not DistributionForm.EditTheDist(NewDist,Sim) then
           begin
             NewDist.Destroy;
             Exit;
           End;

        Inc(NumDists);
        SetLength(DistArray,NumDists);
        DistArray[NumDists-1] := NewDist;
        UpdateScreen;
      End;
    End;

Var Loop, SelIndex: Integer;
begin

  With SelInput do
   Begin
     If ShowModal=mrcancel then exit;

     SelIndex := ListBox1.SelCount;
     If SelIndex = 0 then
          Begin
             MessageDlg('No Distribution is Selected.',mterror,[mbOK],0);
             Exit;
          End;

    For Loop:=0 to ListBox1.Items.Count-1 do
    If ListBox1.Selected[Loop] then
      If SensMode then AddParam(Loop+1)
                  else AddAndEdit(Loop+1);

   End;

end;

Procedure TUncertForm.UpdateScreen;
Var Loop, i: Integer;
    AddStr: String;
Begin
  SensPanel.Visible := SensMode;
  If SensMode then UncertForm.Caption := 'SLAMM Sensitivity Analysis'
              else UncertForm.Caption := 'SLAMM Uncertainty Analysis';

  If SensMode then AddDist.Caption := 'Add Parameter'
              else AddDist.Caption := 'Add Distribution';
  If SensMode then RemoveDist.Caption := 'Remove Parameter'
              else RemoveDist.Caption := 'Remove Distribution';
  EditDist.Visible := not SensMode;
  If SensMode then Label4.Caption := 'Parameter multipliers will be applied to all sub-sites equally.'
              else Label4.Caption := 'Distributions are multipliers and apply to all sub-sites equally.';

  DistBox.Items.Clear;
  If Not SensMode then
    With Sim.UncertSetup do
     For Loop := 0 to NumDists-1 do
      Begin
        AddStr := DistArray[Loop].Name;
        For i := Length(DistArray[Loop].Name) to 41 do
          AddStr := AddSTr + ' ';

        AddStr := AddStr + fixfloattostr(DistArray[Loop].GetValuePointer(0)^,10)+',  ';

        Case DistArray[Loop].DistType of
          Triangular: AddStr := AddStr + 'Tria: ';
          Normal:     AddStr := AddStr + 'Norm: ';
          LogNormal:  AddStr := AddStr + 'LogN: ';
          Uniform:    AddStr := AddStr + 'Unfm: ';
          Else        AddStr := AddStr + 'USER: ';

        End; {case}

        AddStr := AddStr + fixfloattostr(DistArray[Loop].Parm[1],10)+', '+fixfloattostr(DistArray[Loop].Parm[2],10);
        if DistArray[Loop].DistType = Triangular then AddStr := AddStr +', '+ fixfloattostr(DistArray[Loop].Parm[3],10);

        DistBox.Items.Add(AddStr);
      End;

  If SensMode then
   With Sim.UncertSetup do
    For Loop := 0 to NumSens-1 do
     Begin
      DistBox.Items.Add(SensArray[Loop].Name);
     End;

  With Sim.UncertSetup do
    Begin
      UseSeedBox.Checked :=  UseSeed;
      UseRandomSeed.Checked := Not UseSeed;
      UseSeedBoxClick(nil);

      SeedEdit.Text := IntToStr(Seed);
      IterEdit.Text := IntToStr(Iterations);
      GISNumEdit.Text := IntToStr(GIS_Start_Num);

      PctVaryEdit.Text := IntToStr(PctToVary);
    End;
End;

Procedure TUncertForm.NewShell;
Begin
  Sim.UncertSetup.Destroy;
  Sim.UncertSetup := TSLAMM_Uncertainty.Create;
End;

procedure TUncertForm.FormCreate(Sender: TObject);
begin
//  NewShell;
  ShellPath := '';
end;


procedure TUncertForm.HelpButtonClick(Sender: TObject);
begin
   If SensMode then Application.HelpContext(1015)
               else Application.HelpContext(1014);
end;

procedure TUncertForm.EditDistClick(Sender: TObject);
Var Loop, SelectedIndex: Integer;

begin
 If SensMode then Exit;
 
 With Sim.UncertSetup do
  Begin
    SelectedIndex := DistBox.SelCount;
    If SelectedIndex = 0 then
        Begin
           MessageDlg('No Distribution is Selected.',mterror,[mbOK],0);
           Exit;
        End;

    DistributionForm.PDistArr := @DistArray;
    DistributionForm.PNumDists := @NumDists;
    For Loop:=0 to DistBox.Items.Count-1 do
    If DistBox.Selected[Loop] then
      DistributionForm.EditTheDist(DistArray[Loop],Sim);

    UpdateScreen;
  End;
End;

procedure TUncertForm.RemoveDistClick(Sender: TObject);
Var Loop,Loop2: Integer;
    SelectedIndex: Integer;
begin
    SelectedIndex := DistBox.SelCount;
    If SelectedIndex = 0 then
        Begin
           MessageDlg('No Distribution is Selected.',mterror,[mbOK],0);                                             
           Exit;
        End;

  With Sim.UncertSetup do
   For Loop:=DistBox.Items.Count-1 downto 0 do
    If DistBox.Selected[Loop] then
      Begin
        SelectedIndex:=Loop;
        If SensMode then
          Begin
            For Loop2 := SelectedIndex+1 to NumSens-1 do
              SensArray[Loop2-1] := SensArray[Loop2];
            Dec(NumSens);
            SetLength(SensArray,NumSens);
          End
        else
          Begin
            For Loop2 := SelectedIndex+1 to NumDists-1 do
              DistArray[Loop2-1] := DistArray[Loop2];
            Dec(NumDists);
            SetLength(DistArray,NumDists);
          End;
      End;

    UpdateScreen;
End;

procedure TUncertForm.DistBoxKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
   If Key=VK_DELETE then RemoveDistClick(Sender);
   If Key=VK_INSERT then AddDistClick(Sender);
   If Key=VK_RETURN then EditDistClick(Sender);
end;

procedure TUncertForm.UseSeedBoxClick(Sender: TObject);
begin
  SeedLabel.Enabled := UseSeedBox.Checked;
  SeedEdit.Enabled  := UseSeedBox.Checked;
  Sim.UncertSetup.UseSeed := UseSeedBox.Checked;
end;

procedure TUncertForm.ConvInt(Sender: TObject);
Var Conv: Double;
  Result: Integer;

begin
    Val(Trim(TEdit(Sender).Text),Conv,Result);
    With Sim.UncertSetup do
      If Result<>0 then MessageDlg('Incorrect Numerical Format Entered',mterror,[mbOK],0)
                 else begin
                        If Trunc(Conv)=0.0 then Conv:=1.0;
                        case TEdit(Sender).Name[1] of
                           'I': Iterations:=Abs(Trunc(Conv));
                           'S': Seed:=Trunc(Conv);
                           'P': PctToVary := Abs(Trunc(Conv));
                           'G': GIS_Start_Num := Abs(Trunc(Conv));
                         end; {case}
                       end;
    UpdateScreen;
end;

procedure TUncertForm.Close1Click(Sender: TObject);
Var MR: TModalResult;
begin
  MR := MessageDlg('Save Shell before Closing?',mtconfirmation,[mbyes,mbno,mbcancel],0);
  If MR=MRCancel then exit;
  If MR=MRYes then Save1Click(nil);

  NewShell;
  UpdateScreen;
end;

procedure TUncertForm.Save1Click(Sender: TObject);

Begin
  SaveAs1.Click
End;

Procedure TUncertForm.SaveShell;
Var FileStream: TFileStream;
Begin
  FileStream:=TFileStream.Create(ShellPath,fmCreate);

  Sim.UncertSetup.Store(TStream(FileStream));

  FileStream.Destroy;
End;

Procedure TUncertForm.LoadShell;
Var FileStream : TFileStream;
    PC_FileN    :  Array[0..255] of char;
    VersionCheck : String;  {String 10}
    ReadVersionNum: Double;
Begin
  StrPCopy(PC_FileN,ShellPath);
  Try
    FileStream:=TFileStream.Create(PC_FileN,fmOpenRead);
  Except
    MessageDlg(Exception(ExceptObject).Message,mterror,[mbOK],0);
    NewShell;
    Exit;
  End; {Try Except}

  {Check Version #}
  TSRead('ReadVersionNum',ReadVersionNum);

  If ReadVersionNum > VersionNum then
    Raise ESLAMMError.Create('File Version Number (' +  FloatToStrF(ReadVersionNum,ffgeneral,6,6)
     + ') is Greater than Executable Version Number ('+ FloatToStrF(VersionNum    ,ffgeneral,6,6)
     + ').  Please update your version of SLAMM.');

//  If ReadVersionNum<VersionNum then
//     MessageDlg('Converting from '+VersionCheck+' to '+VersionStr,MTInformation,[MbOK],0);

  If ReadVersionNum>VersionNum
    then
       Begin
          MessageDlg('File Version ('+VersionCheck+') is Greater than Executable Version: Unreadable.',mterror,[mbOK],0);
          NewShell;
       End
    else
       Try
         Sim.UncertSetup := TSLAMM_Uncertainty.Load(ReadVersionNum, FileStream, Sim);
       Except
         MessageDlg(Exception(ExceptObject).Message,mterror,[mbOK],0);
         NewShell;
       End;

  FileStream.Destroy;
  UpdateScreen;

End;

procedure TUncertForm.SaveAs1Click(Sender: TObject);
begin
  SaveDialog1.Title:='Select Shell File to Save';
  SaveDialog1.Filter := 'SLAMM UncertShell files (*.sus)|*.sus';
  SaveDialog1.FileName := '';
  If not SaveDialog1.Execute then exit;
  ShellPath := ChangeFileExt(SaveDialog1.FileName, '.sus');
  SaveShell;
end;

procedure TUncertForm.Load1Click(Sender: TObject);
begin
  OpenDialog1.Title:='Select Shell File to Load';
  OpenDialog1.FileName := '';
  OpenDialog1.Filter := 'SLAMM UncertShell files (*.sus)|*.sus';
  If not OpenDialog1.Execute then exit;

  ShellPath := OpenDialog1.FileName;
  LoadShell;

end;



procedure TUncertForm.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
//  Var MR: TModalResult;
begin
//  CanClose := True;
//  MR := MessageDlg('Save Shell before Closing?',mtconfirmation,[mbyes,mbno,mbcancel],0);
//  If MR=MRCancel then CanClose := False;
//  If MR=MRYes then Save1Click(nil);
 {}
end;

procedure TUncertForm.UpBtnClick(Sender: TObject);
Var Distholder:TinputDist;
    SensHolder:TSensParam;
    Loop: Integer;
    SelLoop: Array of Boolean;
begin
 If SensMode
 then With Sim.UncertSetup do
  Begin
    SetLength(SelLoop,NumSens);
    For Loop := 1 to NumSens-1 do
      Begin
        SelLoop[Loop] := DistBox.Selected[Loop];
        If DistBox.Selected[Loop] then
          Begin
            SensHolder := SensArray[Loop];
            SensArray [Loop] := SensArray [Loop-1];
            SensArray [Loop-1] := SensHolder;
          End;
      End;

    UpdateScreen;

    For Loop := 1 to NumSens-1 do
      DistBox.Selected[Loop-1] := SelLoop[Loop];

    SelLoop := Nil;
  End
 else With Sim.UncertSetup do  {Uncert Mode}
  Begin
    SetLength(SelLoop,NumDists);
    For Loop := 1 to NumDists-1 do
      Begin
        SelLoop[Loop] := DistBox.Selected[Loop];
        If DistBox.Selected[Loop] then
          Begin
            DistHolder := DistArray[Loop];
            DistArray[Loop] := DistArray[Loop-1];
            DistArray[Loop-1] := DistHolder;
          End;
      End;

    UpdateScreen;

    For Loop := 1 to NumDists-1 do
      DistBox.Selected[Loop-1] := SelLoop[Loop];

    SelLoop := Nil;
  End;
end;

procedure TUncertForm.DownBtnClick(Sender: TObject);
Var Distholder:TinputDist;
    SensHolder:TSensParam;
    Loop: Integer;
    SelLoop: Array of Boolean;
begin

 If SensMode
 then With Sim.UncertSetup do
  Begin
    SetLength(SelLoop,NumSens);

    For Loop := NumSens-2 downto 0 do
      Begin
        SelLoop[Loop] := DistBox.Selected[Loop];
        If DistBox.Selected[Loop] then
          Begin
            SensHolder := SensArray[Loop];
            SensArray[Loop] := SensArray[Loop+1];
            SensArray[Loop+1] := SensHolder;
          End;
      End;
    UpdateScreen;

    For Loop := NumSens-2 downto 0 do
      DistBox.Selected[Loop+1] := SelLoop[Loop];

    SelLoop := Nil;
  End
 else With Sim.UncertSetup do
  Begin
    SetLength(SelLoop,NumDists);

    For Loop := NumDists-2 downto 0 do
      Begin
        SelLoop[Loop] := DistBox.Selected[Loop];
        If DistBox.Selected[Loop] then
          Begin
            DistHolder := DistArray[Loop];
            DistArray[Loop] := DistArray[Loop+1];
            DistArray[Loop+1] := DistHolder;
          End;
      End;
    UpdateScreen;

    For Loop := NumDists-2 downto 0 do
      DistBox.Selected[Loop+1] := SelLoop[Loop];

    SelLoop := Nil;
  End;
end;

end.

